package com.ruoyi.generator.controller;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.annotation.Anonymous;
import com.ruoyi.common.constant.HttpStatus;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.core.text.Convert;
import com.ruoyi.common.exception.user.ProjectException;
import com.ruoyi.common.utils.*;
import com.ruoyi.common.utils.file.FileUtils;
import com.ruoyi.common.utils.file.ImageUtils;
import com.ruoyi.common.utils.file.OssUtils;
import com.ruoyi.common.utils.ip.IpUtils;
import com.ruoyi.generator.domain.Community.*;
import com.ruoyi.generator.mapper.community.*;
import com.ruoyi.generator.service.*;
import com.ruoyi.generator.vo.*;
import com.ruoyi.system.mapper.SysUserMapper;
import com.ruoyi.system.service.ISysUserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.logging.log4j.util.Strings;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.io.File;
import java.text.ParseException;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 社区文章管理
 *
 * @author fangzhen
 */
@Api(tags = "社区文章管理")
@RestController
@RequestMapping("/community/article")
public class CommunityArticleController extends BaseController {

    private static final Logger log = LoggerFactory.getLogger(CommunityArticleController.class);

    @Autowired
    private ICommunityArticleService communityArticleService;

    @Autowired
    private CommunityClassMapper communityClassMapper;

    @Autowired
    private CommunityArticleCommentMapper communityArticleCommentMapper;

    @Autowired
    private CommunityCommentLikeMapper communityCommentLikeMapper;

    @Autowired
    private CommunityUserCircleMapper communityUserCircleMapper;

    @Autowired
    private CommunityCircleMapper communityCircleMapper;

    @Autowired
    private CommunityCircleAssociationMapper communityCircleAssociationMapper;
    @Autowired
    private CommunityClassCircleMapper communityClassCircleMapper;

    @Autowired
    private CommunityUserInfoMapper communityUserInfoMapper;

    @Autowired
    private ICommunityCollectionService communityCollectionService;

    @Autowired
    private ICommunityArticleRecommendService communityArticleRecommendService;

    @Autowired
    private ICommunityCommentReplyService commentReplyService;

    @Autowired
    private ICommunityPrivacyService communityPrivacyService;

    @Autowired
    private CommunityArticleMapper communityArticleMapper;

    @Autowired
    private CommunityCollectionArticleMapper communityCollectionArticleMapper;

    @Autowired
    private CommunityCollectionMapper communityCollectionMapper;

    @Autowired
    private CommunityTagMapper communityTagMapper;

    @Autowired
    private CommunityCircleExpandMapper communityCircleExpandMapper;

    @Autowired
    private CommunityUserBlockMapper communityUserBlockMapper;

    @Autowired
    private ICommunityCollectionUserService communityCollectionUserService;

    @Autowired
    private ICommunityAccompanyService communityAccompanyService;

    @Autowired
    private ISysUserService userService;

    @Autowired
    private ICommunityArticleTagService communityArticleTagService;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private CommunityReportController communityReportController;

    private static final String CACHE_PREFIX = "article:list:"; // 缓存前缀
    private static final Integer CACHE_EXPIRE_TIME = 30; // 缓存过期时间（分钟）
    @Autowired
    private CommunityUserLikeMapper communityUserLikeMapper;
    @Autowired
    private CommunityLikeMapper communityLikeMapper;
    @Autowired
    private CommunityArticleCollectMapper communityArticleCollectMapper;
    @Autowired
    private CommunityArticleRecommendMapper communityArticleRecommendMapper;
    @Autowired
    private CommunityArticleTagMapper communityArticleTagMapper;

    @Autowired
    private CommunityArticleAtMapper communityArticleAtMapper;

    @Autowired
    private CommunityArticleRecommendMapper recommendMapper;

    @Autowired
    private SysUserMapper sysUserMapper;

    /**
     * 生成唯一的缓存键（基于请求参数）
     */
    public String generateCacheKey(Long userId,
                                   CommunityArticle article,
                                   int pageNum,
                                   int pageSize,
                                   int searchType) {
        StringBuilder keyBuilder = new StringBuilder(CACHE_PREFIX);

        // 添加用户ID
        keyBuilder.append("userId=").append(userId != null ? userId : 0).append(":");

        // 添加文章查询条件（如果有）
        if (article != null) {
            if (article.getClassIds() != null && !article.getClassIds().isEmpty()) {
                // 直接使用逗号连接数字，避免使用String.valueOf()可能产生的"[1]"格式
                StringBuilder classIdsBuilder = new StringBuilder();
                boolean first = true;
                for (Long classId : article.getClassIds()) {
                    if (!first) {
                        classIdsBuilder.append(",");
                    }
                    classIdsBuilder.append(classId);
                    first = false;
                }
                keyBuilder.append("classIds=").append(classIdsBuilder.toString()).append(":");
            }
            if (article.getId() != null) {
                keyBuilder.append("id=").append(article.getId()).append(":");
            }
            if (article.getCircleId() != null) {
                keyBuilder.append("circleId=").append(article.getCircleId()).append(":");
            }
            if (article.getTitle() != null) {
                keyBuilder.append("title=").append(article.getTitle()).append(":");
            }
            // 可以根据需要添加更多条件
            if (article.getUserId() != null) {
                keyBuilder.append("userId=").append(article.getUserId()).append(":");
            }
            if (article.getUserIds() != null) {
                keyBuilder.append("userIds=").append(article.getUserIds()).append(":");
            }
            if (article.getTags() != null) {
                keyBuilder.append("tags=").append(article.getTags()).append(":");
            }
            if (article.getArticleIds() != null) {
                keyBuilder.append("articleIds=").append(article.getArticleIds()).append(":");
            }
            if (article.getPageTagType() != null) {
                keyBuilder.append("pageTagType=").append(article.getPageTagType()).append(":");
            }
            if (article.getNoCollection() != null) {
                keyBuilder.append("noCollection=").append(article.getNoCollection()).append(":");
            }
            if (article.getIsDraft() != null) {
                keyBuilder.append("isDraft=").append(article.getIsDraft()).append(":");
            }
        }

        // 添加分页和搜索类型信息
        keyBuilder.append("page=").append(pageNum).append(":")
                 .append("size=").append(pageSize).append(":")
                 .append("searchType=").append(searchType);

        return keyBuilder.toString();
    }

    /**
     * 清除文章列表相关的所有缓存
     */
    private void clearArticleListCache() {
        try {
            // 获取所有以CACHE_PREFIX开头的缓存键
            Collection<String> keys = redisCache.keys(CACHE_PREFIX + "*");
            if (keys != null && !keys.isEmpty()) {
                // 删除所有匹配的缓存
                redisCache.deleteObject(keys);
                log.info("已清除{}个文章列表缓存", keys.size());
            }
        } catch (Exception e) {
            log.error("清除文章列表缓存失败", e);
        }
    }

    /**
     * 获取文章列表信息
     */
    @ApiOperation("获取文章列表")
    @GetMapping("/list")
    @Anonymous
    public TableDataInfo communityList(CommunityArticle communityArticle, @RequestParam(required = false) Long userId) {
        int pageNum = Convert.toInt(ServletUtils.getParameter("pageNum"), 1);
        int pageSize = Convert.toInt(ServletUtils.getParameter("pageSize"), 10);
        int searchType = Convert.toInt(ServletUtils.getParameter("searchType"), 0);
        TableDataInfo rspData = null;
        if (userId == null) {
            userId = SecurityUtils.getUserId();
        }

        // 生成缓存键
        String cacheKey = generateCacheKey(userId, communityArticle, pageNum, pageSize, searchType);

        try {
            if (searchType != 3) {
// 尝试从缓存中获取数据
                TableDataInfo cachedData = null;
                try {
                    cachedData = redisCache.getCacheObject(cacheKey);
                    if (cachedData != null) {
                        log.info("从缓存中获取文章列表数据, key: {}", cacheKey);
                        return cachedData;
                    }
                } catch (Exception e) {
                    // 如果从缓存获取数据时出错，记录错误并继续从数据库获取
                    log.error("从缓存获取数据失败: {}", e.getMessage(), e);
                    // 清除可能损坏的缓存
                    try {
                        redisCache.deleteObject(cacheKey);
                        log.info("已清除可能损坏的缓存: {}", cacheKey);
                    } catch (Exception ex) {
                        log.error("清除缓存失败: {}", ex.getMessage());
                    }
                }
            }


            // 缓存中没有数据或获取缓存失败，从数据库查询
            log.info("从数据库中获取文章列表数据");
            rspData = new TableDataInfo();
            List<CommunityArticleVo> list = communityArticleService.selectCommunityArticleList(userId, rspData, communityArticle, pageNum, pageSize, searchType);
            rspData.setCode(HttpStatus.SUCCESS);
            rspData.setMsg("查询成功");
            rspData.setRows(list);

            // 将查询结果存入缓存
            try {
                redisCache.setCacheObject(cacheKey, rspData, CACHE_EXPIRE_TIME, TimeUnit.MINUTES);
                log.info("文章列表数据已缓存, key: {}, 过期时间: {}分钟", cacheKey, CACHE_EXPIRE_TIME);
            } catch (Exception e) {
                log.error("缓存数据失败: {}", e.getMessage(), e);
            }
        } catch (Exception e) {
            log.error("获取文章列表失败: {}", e.getMessage(), e);
            throw new ProjectException();
        }
        return rspData;
    }

    /**
     * 发布文章
     */
    @ApiOperation("发布文章")
    @PostMapping("/article")
    @Transactional
    //@Anonymous
    public AjaxResult article(@RequestBody CommunityArticle communityArticle) {
        log.info("communityArticle:{}", JSON.toJSONString(communityArticle));
        try {
            if (communityArticle.getClassIds().isEmpty()) {
                return AjaxResult.error("板块不能为空");
            }

            //判断当前用户有没有被禁言
            SysUser sysUser = sysUserMapper.selectUserById(SecurityUtils.getUserId());
            if (sysUser != null) {
                String userState = sysUser.getUserState();
                Date reportTime = sysUser.getReportTime();
                if ("2".equals(userState) && reportTime != null) {
                    communityReportController.adminReportPunish(SecurityUtils.getUserId());
                    // 获取当前时间
                    Date currentTime = new Date();
                    log.info(String.valueOf(reportTime));
                    log.info(String.valueOf(currentTime));
                    // 检查 reportTime 小于当前时间
                    if (currentTime.before(reportTime)) {
                        // 返回封号的提示
                        return AjaxResult.success("当前用户已被禁言!");
                    }
                }
            }

            String title = communityArticle.getTitle();
            String content = communityArticle.getContent();
            boolean isSensitiveTitle = SensitiveWordUtil.containsSensitiveWord(title);
            if (isSensitiveTitle) {
                return AjaxResult.error(MessageUtils.message("article.title.error"));
            }
            boolean isSensitiveContent = SensitiveWordUtil.containsSensitiveWord(content);
            if (isSensitiveContent) {
                return AjaxResult.error(MessageUtils.message("article.content.error"));
            }

            //检查附近是否是需要加水印
            boolean isWaterMark = communityArticle.isWaterMark();
            boolean isFillWater = communityArticle.isFillWater();
            log.info("isWaterMark:{},isFillWater:{},Images:{}", isWaterMark, isFillWater, communityArticle.getImages());
            if (isWaterMark && null != communityArticle.getImages()) {
                for (CommunityImagesVo image : communityArticle.getImages()) {
                    if (null == image) {
                        throw new RuntimeException("图片上传失败!");
                    }
                }
                log.info("需要做图片/视频修改...");
                //获取用户名
                String nickName = SecurityUtils.getLoginUser().getUser().getNickName();
                // 添加水印
                for (CommunityImagesVo image : communityArticle.getImages()) {
                    String fileType = FileUtils.getFileTypeFromUrl(image.getImages());
                    log.info("fileType:{}", fileType);
                    if ("image".equals(fileType)) {
                        if (image.getImages().contains(".gif") || image.getImages().contains(".GIF")) {
                            continue;
                        }
                        // 本地临时水印文件路径
                        String tempWatermarkFile = System.getProperty("java.io.tmpdir") + File.separator + "watermark_" + FileUtils.getName(image.getImages());
                        String tempSourceFile = System.getProperty("java.io.tmpdir") + File.separator + "source_" + FileUtils.getName(image.getImages());
                        OssUtils.downloadFile(image.getImages().replace("http://file.iciyuanshidai.com/", ""), tempSourceFile);
                        ImageUtils.addWatermark(
                                isFillWater,
                                image.getImages().replace("http://file.iciyuanshidai.com/", ""),
                                new File(tempSourceFile),
                                new File(tempWatermarkFile),
                                "@" + nickName,
                                "次元时代-ACG爱好者社区"
                        );
                        // 上传加水印的文件到OSS
                        String fileNameResult = OssUtils.uploadFile(tempWatermarkFile);
                        image.setImages(fileNameResult.replace("https://cysd.oss-cn-shanghai.aliyuncs.com", "http://file.iciyuanshidai.com/"));
                    }


                    if ("video".equals(fileType)) {
                        String tempThumbnailFile = System.getProperty("java.io.tmpdir") + File.separator + "thumbnail.jpg";

                        boolean screenshotSuccess = FileUtils.screenShots(image.getImages(), tempThumbnailFile, "00:00:01");
                        if (!screenshotSuccess) {
                            throw new RuntimeException("生成视频缩略图失败");
                        }

                        // 上传缩略图到OSS
                        String thumbnailPath = OssUtils.uploadFile(tempThumbnailFile);
                        image.setCompressedImages(thumbnailPath.replace("https://cysd.oss-cn-shanghai.aliyuncs.com", "http://file.iciyuanshidai.com/"));
                        // 删除临时缩略图文件
                        new File(tempThumbnailFile).delete();
                    }
                }
            }

            communityArticleService.insertCommunityArticle(communityArticle);

            // 清除文章列表缓存，确保新发布的文章能够立即显示
            clearArticleListCache();
            log.info("文章发布成功，已清除文章列表缓存");
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success("文章发布成功!");
    }

    /**
     * 编辑文章
     */
    @ApiOperation("编辑文章")
    @PostMapping("/editArticle")
    @Transactional
    //@Anonymous
    public AjaxResult editArticle(@RequestBody CommunityArticle communityArticle) {
        try {
            log.info("communityArticle:{}", communityArticle);
            if (communityArticle.getClassIds().isEmpty()) {
                return AjaxResult.error("板块不能为空");
            }
            if (communityArticle.getId() == null) {
                return AjaxResult.error("文章ID不能为空");
            }
            String title = communityArticle.getTitle();
            String content = communityArticle.getContent();
            boolean isSensitiveTitle = SensitiveWordUtil.containsSensitiveWord(title);
            if (isSensitiveTitle) {
                return AjaxResult.error(MessageUtils.message("article.title.error"));
            }
            boolean isSensitiveContent = SensitiveWordUtil.containsSensitiveWord(content);
            if (isSensitiveContent) {
                return AjaxResult.error(MessageUtils.message("article.content.error"));
            }

            communityArticleService.editCommunityArticle(communityArticle);

            // 清除文章列表缓存，确保编辑后的文章能够立即显示
            clearArticleListCache();
            log.info("文章编辑成功，已清除文章列表缓存");
        } catch (Exception e) {
            // e.printStackTrace();
            log.info(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success("文章编辑成功!");
    }

    /**
     * 删除文章
     */
    @ApiOperation("删除文章")
    @DeleteMapping("/article")
    @Transactional
    //@Anonymous
    public AjaxResult article(String id) {
        if (Strings.isEmpty(id)) {
            return AjaxResult.error("参数异常!");
        }
        try {
            //文章删除
            CommunityArticle communityArticle = new CommunityArticle();
            communityArticle.setIsDelete(true);
            boolean result1 = communityArticleService.update(communityArticle, new UpdateWrapper<CommunityArticle>().eq("id", id));
            //文章合集删除
            int result2 = communityCollectionArticleMapper.update(null, new UpdateWrapper<CommunityCollectionArticle>()
                    .eq("article_id", id)
                    .set("is_delete", true));
            //文章关联的标签删除
            communityArticleTagService.update(new UpdateWrapper<CommunityArticleTag>()
                    .set("is_delete", true)
                    .set("update_time", DateUtils.parseDate(DateUtils.getTime()))
                    .set("update_by", SecurityUtils.getUserId())
                    .eq("article_id", id));
            if (result1 && result2 >= 0) {
                // 清除文章列表缓存，确保删除的文章不再显示
                clearArticleListCache();
                log.info("文章删除成功，已清除文章列表缓存");
                return success("删除成功!");
            }
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }
        return error("删除失败!");
    }


    @GetMapping("/class")
    @ApiOperation("获取文章分类列表")
    //@Anonymous
    public AjaxResult getClassList() {

        List<CommunityClass> communityClasses = null;
        //获取用户信息
        CommunityUserInfoVo communityUserInfoVo = null;
        try {
            Long userId = SecurityUtils.getLoginUser().getUserId();
            String classId = communityArticleService.selectCommunityUserInfoById(userId, true).getClassId();

            if (classId != null && !classId.isEmpty()) {
                List<Long> sortOrder = Arrays.stream(classId.split(","))
                        .map(String::trim)
                        .map(Long::parseLong)
                        .collect(Collectors.toList());

                // 假设 allCommunityClasses 是从数据库查询出来的结果
                List<CommunityClass> allCommunityClasses = communityClassMapper.selectList(new QueryWrapper<CommunityClass>().orderByAsc("sort"));

                // 将 allCommunityClasses 按照 sortOrder 分组并排序

                Map<Long, CommunityClass> classMap = allCommunityClasses.stream().collect(Collectors.toMap(CommunityClass::getId, c -> c));

                List<CommunityClass> sortedCommunityClasses = new ArrayList<>();
                // 按 sortOrder 先添加在列表中的元素
                for (Long order : sortOrder) {
                    if (classMap.containsKey(order)) {
                        sortedCommunityClasses.add(classMap.get(order));
                    }
                }
                // 保持原有顺序添加不在 sortOrder 中的元素
                List<CommunityClass> remainingClasses = allCommunityClasses.stream()
                        .filter(c -> !sortOrder.contains(c.getId()))
                        .collect(Collectors.toList());

                sortedCommunityClasses.addAll(remainingClasses);
                return AjaxResult.success(sortedCommunityClasses);
            } else {
                communityClasses = communityClassMapper.selectList(new QueryWrapper<CommunityClass>().orderByAsc("sort"));

            }
        } catch (Exception e) {
            //e.printStackTrace();
            log.info(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success(communityClasses);
    }


    @GetMapping("/circleList")
    @ApiOperation("获取文章圈子列表")
    //@Anonymous
    public AjaxResult getCircleList() {
        List<CommunityCircleExpand> communityCircleExpands = null;
        Long userId = SecurityUtils.getUserId();
        JSONArray sumArray = new JSONArray();
        try {
            communityCircleExpands = communityCircleExpandMapper.selectList(new QueryWrapper<CommunityCircleExpand>());
            for (CommunityCircleExpand communityCircleExpand : communityCircleExpands) {
                String circleName = communityCircleExpand.getCircleName();
                Long id = communityCircleExpand.getId();
                JSONObject oneObject = new JSONObject(); // 每次循环创建新的 JSONObject
                oneObject.put("oneName", circleName);
                oneObject.put("oneId", communityCircleExpand.getId());
                JSONArray twoArray = new JSONArray();
                List<CommunityCircle> classNameByCircleId = communityClassCircleMapper.getClassNameByCircleId(userId, id);
                // 将 List<CommunityCircle> 转换为 JSONArray
                for (CommunityCircle communityCircle : classNameByCircleId) {
                    JSONObject twoObject = new JSONObject();
                    twoObject.put("twoId", communityCircle.getId());
                    twoObject.put("twoName", communityCircle.getCircleName());
                    twoObject.put("twoImg", communityCircle.getImageUrl());
                    twoObject.put("isCare", communityCircle.getIsCare());
                    twoObject.put("heat", communityCircle.getHeat());
                    twoArray.add(twoObject);
                }
                oneObject.put("one", twoArray);
                sumArray.add(oneObject); // 添加到 sumArray
            }
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success(sumArray);
    }


    @ApiOperation("文章评论")
    @PostMapping("/comment")
    @Transactional
    public AjaxResult comment(@RequestBody CommunityArticleComment communityArticleComment) throws ParseException {
        if (null == communityArticleComment.getArticleId()) {
            return AjaxResult.error("文章不存在或异常!");
        }
        //判断当前用户有没有被禁言
        SysUser sysUser = sysUserMapper.selectUserById(SecurityUtils.getUserId());
        if (sysUser != null) {
            String userState = sysUser.getUserState();
            Date reportTime = sysUser.getReportTime();
            if ("2".equals(userState) && reportTime != null) {
                communityReportController.adminReportPunish(SecurityUtils.getUserId());
                // 获取当前时间
                Date currentTime = new Date();
                // 检查 reportTime 小于当前时间
                if (currentTime.before(reportTime)) {
                    // 返回封号的提示
                    return AjaxResult.success("当前用户已被禁言!");
                }
            }
        }

        //判断敏感词汇
        boolean isSensitive = SensitiveWordUtil.containsSensitiveWord(communityArticleComment.getContent());
        if (isSensitive) {
            return AjaxResult.error(MessageUtils.message("article.user.comment"));
        }

        Long userId = SecurityUtils.getUserId();
        try {
            //调用方法去判断当前评论的用户 是不是文章作者的关注列表
            Boolean aBoolean = communityArticleService.checkCommentPermission(userId, null, communityArticleComment.getArticleId());
            communityArticleComment.setUserId(SecurityUtils.getUserId());
            if (!aBoolean) {  // 当 aBoolean 为 false 时，进入这个条件
                return AjaxResult.success("您没有权限评论此文章，因为作者只允许关注的人评论!");
            }
            //新增评论
            communityArticleService.sendComment(communityArticleComment);
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success();
    }

    @ApiOperation("删除文章评价")
    @DeleteMapping("/comment")
    @Transactional
    public AjaxResult deleteComment(@RequestBody CommunityArticleComment communityArticleComment) {
        try {
            //获取评论信息
            CommunityArticleComment articleComment = communityArticleCommentMapper.selectById(communityArticleComment.getId());
            Long articleId = articleComment.getArticleId();
            Long userId = SecurityUtils.getUserId();
            if (articleId != null) {
                CommunityArticle communityArticle = communityArticleService.getById(articleId);
                if (communityArticle.getCreateBy().equals(userId)) {
                    //文章作者可以删除文章内的评论
                    //删除评论
                    communityArticleService.deleteComment(articleComment);
                    return AjaxResult.success("删除评论成功!");
                }
            }

            if (!articleComment.getCreateBy().equals(userId)) {
                return AjaxResult.error("不允许删除别人的评论!");
            }
            //删除评论
            communityArticleService.deleteComment(articleComment);
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success("删除评论成功!");
    }

    /**
     * 文章点赞
     */
    @ApiOperation("文章点赞")
    @PostMapping("/like")
    @Transactional
    //@Anonymous
    public AjaxResult like(@RequestBody CommunityLike communityLike) {
        if (Objects.isNull(communityLike.getArticleId()) || Objects.isNull(communityLike.getUserId())) {
            return AjaxResult.error("文章或用户信息异常!");
        }
        boolean isSuccess = true;
        String message = "";
        try {
            CommunityLike like = communityArticleService.selectCommunityLikeById(communityLike);
            if (Objects.isNull(like)) {
                //新增点赞
                communityLike.setCreateBy(String.valueOf(SecurityUtils.getLoginUser().getUserId()));
                communityLike.setUpdateBy(String.valueOf(SecurityUtils.getLoginUser().getUserId()));
                communityLike.setCreateTime(DateUtils.parseDate(DateUtils.getTime()));
                communityLike.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
                isSuccess = communityArticleService.insertCommunityLikeById(communityLike);
                message = "感谢点赞";
            } else {
                //删除点赞
                isSuccess = communityArticleService.deleteCommunityLikeById(like);
                message = "取消点赞";
            }

            if (!isSuccess) {
                message = "操作失败!";
                return AjaxResult.error(message);
            }

//            redisCache.setCacheList(CacheConstants.COMMUNITY_LIKE, communityLikeMapper.selectList(new QueryWrapper<CommunityLike>()));
//            redisCache.expire(CacheConstants.COMMUNITY_LIKE, 24, TimeUnit.HOURS);
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success(message);
    }

    /**
     * 文章推荐值
     */
    @ApiOperation("新增/取消推荐值")
    @PostMapping("/recommend")
    @Transactional
    //@Anonymous
    public AjaxResult recommend(@RequestBody CommunityArticleRecommend articleRecommend) {
        if (Objects.isNull(articleRecommend.getArticleId())) {
            return AjaxResult.error("参数异常!");
        }
        boolean isSuccess = true;
        String message = "";
        Long userId = SecurityUtils.getUserId();
        try {
            CommunityArticleRecommend recommend = communityArticleRecommendService.getOne(new QueryWrapper<CommunityArticleRecommend>()
                    .eq("article_id", articleRecommend.getArticleId())
                    .eq("user_id", userId).and((wrapper) -> {
                        wrapper.ne("is_delete", true).or().isNull("is_delete");
                    }));
            if (Objects.isNull(recommend)) {
                //新增推荐值
                recommend = new CommunityArticleRecommend();
                recommend.setArticleId(articleRecommend.getArticleId());
                recommend.setCreateBy(userId);
                recommend.setUserId(userId);
                recommend.setDelete(false);
                recommend.setCreateTime(DateUtils.parseDate(DateUtils.getTime()));
                isSuccess = communityArticleRecommendService.save(recommend);
                message = "感谢推荐";
            } else {
                //删除点赞
                recommend.setDelete(true);
                recommend.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
                recommend.setUpdateBy(userId);
                isSuccess = communityArticleRecommendService.updateById(recommend);
                message = "取消推荐";
            }

            if (!isSuccess) {
                message = "操作失败!";
                return AjaxResult.error(message);
            }

//            redisCache.setCacheList(CacheConstants.COMMUNITY_ARTICLE_RECOMMEND, communityArticleRecommendMapper.selectList(new QueryWrapper<CommunityArticleRecommend>().eq("is_delete", false)));
//            redisCache.expire(CacheConstants.COMMUNITY_ARTICLE_RECOMMEND, 24, TimeUnit.HOURS);
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success(message);
    }


    /**
     * 获取文章列表信息
     */
    @ApiOperation("文章收藏")
    @PostMapping("/collect")
    @Transactional
    //@Anonymous
    public AjaxResult collect(@RequestBody CommunityArticleCollect communityArticleCollect) {
        if (Objects.isNull(communityArticleCollect.getArticleId()) || Objects.isNull(communityArticleCollect.getUserId())) {
            return AjaxResult.error("文章或用户信息异常!");
        }
        boolean isSuccess = true;
        String message = "";

        try {
            CommunityArticleCollect collect = communityArticleService.selectCommunityCollectById(communityArticleCollect);
            if (Objects.isNull(collect) || Objects.isNull(collect.getId())) {
                //新增收藏
                communityArticleCollect.setCreateTime(DateUtils.parseDate(DateUtils.getTime()));
                communityArticleCollect.setCreateBy(String.valueOf(SecurityUtils.getUserId()));
                isSuccess = communityArticleService.insertCommunityCollectById(communityArticleCollect);
                message = "收藏成功";
            } else {
                //删除收藏
                isSuccess = communityArticleService.deleteCommunityCollectById(collect);
                message = "取消收藏";
            }

            if (!isSuccess) {
                message = "操作失败!";
                return AjaxResult.error(message);
            }

//            redisCache.setCacheList(CacheConstants.COMMUNITY_COLLECT, communityArticleCollectMapper.selectList(new QueryWrapper<CommunityArticleCollect>()));
//            redisCache.expire(CacheConstants.COMMUNITY_COLLECT, 24, TimeUnit.HOURS);
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success(message);
    }


    /**
     * 评论点赞
     */
    @ApiOperation("评论点赞")
    @PostMapping("/commentLike")
    @Transactional
    //@Anonymous
    public AjaxResult commentLike(@RequestBody CommunityCommentLike commentLike) {
        if (Objects.isNull(commentLike.getCommentId())) {
            return AjaxResult.error("评论或用户信息异常!");
        }
        Long userId = SecurityUtils.getLoginUser().getUserId();
        String message = "";

        try {
            //评论点赞
            //查询该评论是否已点赞
            QueryWrapper<CommunityCommentLike> queryWrapper = null;
            if (commentLike.isReply()) {
                //回复的点赞
                queryWrapper = new QueryWrapper<CommunityCommentLike>()
                        .eq("reply_id", commentLike.getCommentId())
                        .eq("user_id", userId);
                commentLike.setReplyId(commentLike.getCommentId());
                commentLike.setCommentId(null);
            } else {
                //评论的点赞
                queryWrapper = new QueryWrapper<CommunityCommentLike>()
                        .eq("comment_id", commentLike.getCommentId())
                        .eq("user_id", userId);
            }

            CommunityCommentLike communityCommentLike = communityCommentLikeMapper.selectOne(queryWrapper);
            if (Objects.isNull(communityCommentLike) || Objects.isNull(communityCommentLike.getId())) {
                //新增点赞
                commentLike.setCreateBy(userId);
                commentLike.setUserId(userId);
                commentLike.setCreateTime(DateUtils.parseDate(DateUtils.getTime()));
                communityCommentLikeMapper.insert(commentLike);
                message = "感谢点赞";
            } else {
                //删除点赞
                communityCommentLikeMapper.deleteById(communityCommentLike);
                message = "取消点赞";
            }
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success(message);
    }


    @ApiOperation("获取圈子信息")
    @GetMapping("/circle")
    //@Anonymous
    public AjaxResult circle(@RequestParam("classId") String classId) {
        if (Strings.isBlank(classId)) {
            return AjaxResult.error("板块信息有误!");
        }
        List<CommunityCircleVo> communityCircles = null;
        try {
            communityCircles = communityArticleService.selectCommunityCircleList(classId);
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success(communityCircles);
    }

    @ApiOperation("我的圈子")
    @GetMapping("/myCircle")
    //@Anonymous
    public AjaxResult myCircle(@RequestParam(required = false) String circleName) {
        Long userId = SecurityUtils.getLoginUser().getUserId();

        List<CommunityCircle> communityCircles = null;
        try {
            List<CommunityUserCircle> communityUserCircles = communityUserCircleMapper.selectList(new QueryWrapper<CommunityUserCircle>()
                    .eq("user_id", userId)
                    .and((wrapper) -> {
                        wrapper.ne("is_delete", 1).or().isNull("is_delete");
                    }));

            //获取圈子的IDS
            List<Long> circleIds = new ArrayList<>();
            for (CommunityUserCircle communityUserCircle : communityUserCircles) {
                circleIds.add(communityUserCircle.getCircleId());
            }

            if (!circleIds.isEmpty()) {
                communityCircles = communityCircleMapper.selectList(new QueryWrapper<CommunityCircle>()
                        .in("id", circleIds)
                        .like("circle_name", circleName)
                        .and((wrapper) -> {
                            wrapper.ne("is_delete", 1).or().isNull("is_delete");
                        }));
            }

            //为空返回空值
            if (communityCircles == null) {
                return AjaxResult.success("圈子为空");
            }

            for (CommunityCircle communityCircle : communityCircles) {
                Long id = communityCircle.getId();
                Long size = communityUserCircleMapper.selectCount(new QueryWrapper<CommunityUserCircle>()
                        .eq("circle_id", id)
                        .and((wrapper) -> {
                            wrapper.ne("is_delete", 1).or().isNull("is_delete");
                        }));
                communityCircle.setHeat(size);
            }

        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success("获取成功!", communityCircles);
    }

    @ApiOperation("编辑圈子")
    @PostMapping("/editCircle")
    @Transactional
    //@Anonymous
    public AjaxResult editCircle(@RequestBody CommunityCircleEditVo communityCircleEditVo) {
        if (Objects.isNull(communityCircleEditVo.getOneId())) {
            return AjaxResult.error("圈子一级ID不能为空");
        }
        if (Objects.isNull(communityCircleEditVo.getType())) {
            return AjaxResult.error("编辑类型不能为空");
        }

        //圈子
        CommunityCircle communityCircle = new CommunityCircle();
        //关联关系
        CommunityCircleAssociation communityCircleAssociation = new CommunityCircleAssociation();

        String type = String.valueOf(communityCircleEditVo.getType());
        try {

            switch (type) {
                case "1":
                    //查询圈子
                    CommunityCircle circle = communityCircleMapper.selectOne(new QueryWrapper<CommunityCircle>()
                            .eq("circle_name", communityCircleEditVo.getTwoName())
                            .and((wrapper) -> {
                                wrapper.ne("is_delete", 1).or().isNull("is_delete");
                            }));
                    if (circle != null){
                        return AjaxResult.success("圈子名称已经存在无法添加");
                    }
                    //增加圈子
                    communityCircle.setCircleName(communityCircleEditVo.getTwoName());
                    communityCircle.setImageUrl(communityCircleEditVo.getTwoImg());
                    communityCircle.setCreateBy(SecurityUtils.getLoginUser().getUserId());
                    communityCircle.setCreateTime(DateUtils.parseDate(DateUtils.getTime()));
                    communityCircleMapper.insert(communityCircle);

                    //增加圈子的关联关系
                    communityCircleAssociation.setCircleId(communityCircle.getId());
                    communityCircleAssociation.setExpandId(communityCircleEditVo.getOneId());
                    communityCircleAssociation.setCreateBy(SecurityUtils.getLoginUser().getUserId());
                    communityCircleAssociation.setCreateTime(DateUtils.parseDate(DateUtils.getTime()));
                    communityCircleAssociationMapper.insert(communityCircleAssociation);

                    return AjaxResult.success("添加成功");
                case "2":
                    //查询圈子
                    CommunityCircle editCircle = communityCircleMapper.selectOne(new QueryWrapper<CommunityCircle>()
                            .eq("id", communityCircleEditVo.getTwoId())
                            .and((wrapper) -> {
                                wrapper.ne("is_delete", 1).or().isNull("is_delete");
                            }));

                    //编辑圈子
                    if (!Objects.isNull(communityCircleEditVo.getTwoName())) {
                        editCircle.setCircleName(communityCircleEditVo.getTwoName());
                    }
                    if (!Objects.isNull(communityCircleEditVo.getTwoImg())) {
                        editCircle.setImageUrl(communityCircleEditVo.getTwoImg());
                    }

                    editCircle.setUpdateBy(SecurityUtils.getLoginUser().getUserId());
                    editCircle.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
                    communityCircleMapper.updateById(editCircle);

                    //查询关联关系ID
                    communityCircleAssociation = communityCircleAssociationMapper.selectOne(new QueryWrapper<CommunityCircleAssociation>()
                            .eq("circle_id", communityCircleEditVo.getTwoId()));

                    //修改圈子的关联关系
                    communityCircleAssociation.setExpandId(communityCircleEditVo.getOneId());
                    communityCircleAssociation.setUpdateBy(SecurityUtils.getLoginUser().getUserId());
                    communityCircleAssociation.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
                    communityCircleAssociationMapper.updateById(communityCircleAssociation);

                    return AjaxResult.success("编辑成功");
                case "3":
                    //删除圈子 关联关系不用删 直接给圈子变更
                    CommunityCircle deleteCircle = communityCircleMapper.selectOne(new QueryWrapper<CommunityCircle>()
                            .eq("id", communityCircleEditVo.getTwoId())
                            .and((wrapper) -> {
                                wrapper.ne("is_delete", 1).or().isNull("is_delete");
                            }));

                    deleteCircle.setIsDelete(true);
                    deleteCircle.setUpdateBy(SecurityUtils.getLoginUser().getUserId());
                    deleteCircle.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
                    communityCircleMapper.updateById(deleteCircle);
                    return AjaxResult.success("删除成功");
                default:
                    return AjaxResult.error("无效的操作类型");
            }
        } catch (Exception e) {
            log.info(e.getMessage());
            return AjaxResult.error("操作失败");

        }
    }


    @ApiOperation("收藏/取关圈子")
    @PostMapping("/circle")
    @Transactional
    //@Anonymous
    public AjaxResult circle(@RequestBody CommunityUserCircle communityUserCircle) {
        if (Objects.isNull(communityUserCircle.getCircleId())) {
            return AjaxResult.error("关注圈子异常,请刷新页面重试!");
        }
        Long userId = SecurityUtils.getLoginUser().getUserId();
        String msg = "";
        try {
            CommunityUserCircle userCircle = communityUserCircleMapper
                    .selectOne(new QueryWrapper<CommunityUserCircle>()
                            .eq("circle_id", communityUserCircle.getCircleId())
                            .eq("user_id", userId)
                            .eq("is_delete", 0));

            if (Objects.isNull(userCircle) || Objects.isNull(userCircle.getId())) {
                //如果未找到数据,则进行关注
                userCircle = new CommunityUserCircle();
                userCircle.setCircleId(communityUserCircle.getCircleId());
                userCircle.setUserId(userId);
                userCircle.setCreateBy(userId);
                userCircle.setCreateTime(DateUtils.parseDate(DateUtils.getTime()));
                communityUserCircleMapper.insert(userCircle);
                msg = "已关注";
            } else {
                //更新删除标识
                userCircle.setDelete(true);
                userCircle.setUpdateBy(userId);
                userCircle.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
                communityUserCircleMapper.updateById(userCircle);
                msg = "已取关";
            }
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success(msg);
    }

    /**
     * 关注用户
     *
     * @param likeUserId 被关注人id
     * @return 当前关注信息
     */

    @ApiOperation("关注/取关用户")
    @PostMapping("/likeUser")
    @Transactional
    public AjaxResult likeUser(@RequestParam("likeUserId") Long likeUserId) {
        if (Objects.isNull(likeUserId)) {
            return AjaxResult.error("参数失效!");
        }
        //无法关注自己
        Long userId = SecurityUtils.getLoginUser().getUserId();
        if (userId.equals(likeUserId)) {
            return AjaxResult.error("无法关注自己!");
        }

        String msg = "";
        CommunityUserLike communityUserLike = communityArticleService.addOrDeleteUserLike(likeUserId);
        try {
            if (Objects.isNull(communityUserLike) || Objects.isNull(communityUserLike.getId())) {
                msg = "取消关注";
            } else {
                msg = "已关注";
            }
//            redisCache.setCacheList(CacheConstants.COMMUNITY_USER_LIKE, communityUserLikeMapper.selectList(new QueryWrapper<CommunityUserLike>()));
//            redisCache.expire(CacheConstants.COMMUNITY_USER_LIKE, 24, TimeUnit.HOURS);
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success(msg);
    }

    @ApiOperation("获取用户信息")
    @GetMapping("/userInfo")
    @Anonymous
    public AjaxResult userInfo(@RequestParam(required = false) Long userId, boolean isToken) {
        if (Objects.isNull(userId)) {
            userId = SecurityUtils.getLoginUser().getUserId();

            //是本人刷新下IP地址
            SysUser sysUser = new SysUser();
            sysUser.setUserId(userId);
            sysUser.setLoginIp(IpUtils.getIpAddr());
            sysUser.setLoginDate(DateUtils.getNowDate());
            userService.updateUserProfile(sysUser);
        }

        CommunityUserInfoVo communityUserInfoVo = null;
        try {
            communityUserInfoVo = communityArticleService.selectCommunityUserInfoById(userId, isToken);
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success(communityUserInfoVo);
    }


    @ApiOperation("搜索用户列表信息")
    @GetMapping("/userList")
    public AjaxResult userList(String SearchValue) {
        if (Objects.isNull(SearchValue)) {
            return AjaxResult.error();
        }
        Long userId = SecurityUtils.getLoginUser().getUserId();
        List<CommunityUserInfoVo> communityUserInfoVos = null;
        try {
            communityUserInfoVos = communityArticleService.selectUserBySearchValue(SearchValue);

            List<CommunityUserInfoVo> communityUserInfoVos_copy = new ArrayList<>(communityUserInfoVos);
            for (CommunityUserInfoVo communityUserInfoVo : communityUserInfoVos_copy) {
                communityUserInfoVo.getUserId();
                List<CommunityUserBlock> communityUserBlocks = communityUserBlockMapper.selectList(new QueryWrapper<CommunityUserBlock>()
                        .eq("user_id", userId)
                        .eq("peer_id", communityUserInfoVo.getUserId()));
                for (CommunityUserBlock userBlock : communityUserBlocks) {
                    if (userBlock.isBlock()) {
                        communityUserInfoVo.setBlock(userBlock.isBlock());
                        break;
                    }
                }
            }
        } catch (Exception e) {
            // e.printStackTrace();
            log.info(e.getMessage());
            throw new ProjectException();
        }


        return AjaxResult.success(communityUserInfoVos);
    }

    @ApiOperation("搜索黑名单")
    @GetMapping("/userListBlock")
    public AjaxResult userListBlock(String SearchValue) {

        int pageNum = Convert.toInt(ServletUtils.getParameter("pageNum"), 1);
        int pageSize = Convert.toInt(ServletUtils.getParameter("pageSize"), 10);
        int offset = (pageNum - 1) * pageSize;
        Long userId = SecurityUtils.getLoginUser().getUserId();
        List<CommunityUserInfoVo> communityUserInfoVos = null;
        try {
            communityUserInfoVos = communityArticleService.selectBlockUserBySearchValue(SearchValue,userId,offset,pageSize);
            List<CommunityUserInfoVo> communityUserInfoVos_copy = new ArrayList<>(communityUserInfoVos);
            for (CommunityUserInfoVo communityUserInfoVo : communityUserInfoVos_copy) {
                communityUserInfoVo.setBlock(true);
            }
        } catch (Exception e) {
            // e.printStackTrace();
            log.info(e.getMessage());
            throw new ProjectException();
        }


        return AjaxResult.success(communityUserInfoVos);
    }

    @ApiOperation("根据条件查询账户")
    @GetMapping("/selectUserList")
    public AjaxResult selectUserList(String SearchValue) {
        if (Objects.isNull(SearchValue)) {
            return AjaxResult.error();
        }
        List<CommunityUserInfoVo> communityUserInfoVos = null;
        try {
            communityUserInfoVos = communityArticleService.selectUsersByAccountOrName(SearchValue);
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success(communityUserInfoVos);
    }


    @ApiOperation("获取关注列表")
    @GetMapping("/likeUserList")
    @Anonymous
    public AjaxResult likeUserList(String userName, @RequestParam(required = false) Long userId) {
        int pageNum = Convert.toInt(ServletUtils.getParameter("pageNum"), 1);
        int pageSize = Convert.toInt(ServletUtils.getParameter("pageSize"), 10);
        int searchType = Convert.toInt(ServletUtils.getParameter("searchType"), 0);
        List<CommunityUserLikeVo> communityUserLikeVos = null;
        if (userId == null) {
            userId = SecurityUtils.getLoginUser().getUserId();
        }
        try {
            communityUserLikeVos = communityArticleService.selectCommunityUserLikeList(userId, userName, pageNum, pageSize, searchType);
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success(communityUserLikeVos);
    }

    @ApiOperation("获取粉丝列表")
    @GetMapping("/userFans")
    public AjaxResult userFans(Long userId, String userName) {
        if (Objects.isNull(userId)) {
            userId = SecurityUtils.getLoginUser().getUserId();
        }
        int pageNum = Convert.toInt(ServletUtils.getParameter("pageNum"), 1);
        int pageSize = Convert.toInt(ServletUtils.getParameter("pageSize"), 10);
        int searchType = Convert.toInt(ServletUtils.getParameter("searchType"), 0);
        List<CommunityUserLikeVo> communityUserLikeVos = null;

        try {
            communityUserLikeVos = communityArticleService.selectUserFansList(userId, userName, pageNum, pageSize, searchType);
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success(communityUserLikeVos);
    }

    @ApiOperation("互相关注列表")
    @GetMapping("/mutualAttention")
    public AjaxResult mutualAttention(String userName) {
        int pageNum = Convert.toInt(ServletUtils.getParameter("pageNum"), 1);
        int pageSize = Convert.toInt(ServletUtils.getParameter("pageSize"), 10);
        int searchType = Convert.toInt(ServletUtils.getParameter("searchType"), 0);

        List<CommunityUserLikeVo> communityUserLikeVos = null;
        try {
            communityUserLikeVos = communityArticleService.selectMutualAttention(SecurityUtils.getLoginUser().getUserId(), userName, pageNum, pageSize, searchType);
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success(communityUserLikeVos);
    }

    @ApiOperation("发送用户验证码")
    @GetMapping("/sendSms")
    //todo
    public AjaxResult sendSms() {
        return AjaxResult.success();
    }

    /**
     * 更新用户背景图片
     *
     * @param backImage 背景图url
     * @return 是否成功
     */
    @ApiOperation("更新用户背景图片")
    @PostMapping("/uploadBackImage")
    @Transactional
    public AjaxResult uploadBackImage(@RequestParam("backImage") String backImage) {
        if (backImage == null) {
            return AjaxResult.error("背景图不能为空");
        }
        Long userId = SecurityUtils.getLoginUser().getUserId();
        try {
            CommunityUserInfo communityUserInfo = communityUserInfoMapper.selectOne(new QueryWrapper<CommunityUserInfo>().eq("user_id", userId));
            if (Objects.isNull(communityUserInfo) || Objects.isNull(communityUserInfo.getId())) {
                return AjaxResult.error("系统出错");
            }
            communityUserInfo.setBackImage(backImage);
            communityUserInfoMapper.updateById(communityUserInfo);
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success("背景图片上传成功!");
    }

    /**
     * 用户合集
     *
     * @return 是否成功
     */
    @ApiOperation("用户合集")
    @GetMapping("/userCollection")
    @Anonymous
    public AjaxResult userCollection(Long userId, Long articleId) {
        if (Objects.isNull(userId)) {
            return AjaxResult.error("参数异常!");
        }
        int pageNum = Convert.toInt(ServletUtils.getParameter("pageNum"), 1);
        int pageSize = Convert.toInt(ServletUtils.getParameter("pageSize"), 10);
        int searchType = Convert.toInt(ServletUtils.getParameter("searchType"), 1);
//        List<CommunityCollection> collections = communityCollectionService.list(new QueryWrapper<CommunityCollection>()
//                .eq("user_id", userId)
//                .ne("is_delete", true)
//                .or()
//                .isNull("is_delete"));

        List<CommunityCollectionVo> communityCollectionVos = null;
        try {
            communityCollectionVos = communityCollectionService.selectUserCollection(userId, articleId, pageNum, pageSize, searchType);
        } catch (Exception e) {
            e.printStackTrace();
            log.info(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success(communityCollectionVos);
    }


    /**
     * 用户新增合集
     *
     * @param communityCollection 新增合集数据
     * @return 是否成功
     */
    @ApiOperation("用户新增合集")
    @PostMapping("/createCollection")
    @Transactional
    public AjaxResult createCollection(@RequestBody CommunityCollection communityCollection) {

        if (Objects.isNull(communityCollection.getCollectionName())) {
            return AjaxResult.error("合集名称不能为空");
        }

        if (Objects.isNull(communityCollection.getImages())) {
            return AjaxResult.error("合集图片不能为空");
        }

        try {
            CommunityCollection collection = communityCollectionService
                    .getOne(new QueryWrapper<CommunityCollection>()
                            .eq("collection_name", communityCollection.getCollectionName())
                            .eq("user_id", SecurityUtils.getLoginUser().getUserId())
                            .and((wrapper) -> {
                                wrapper.ne("is_delete", true)
                                        .or()
                                        .isNull("is_delete");
                            }));

            if (!Objects.isNull(collection)) {
                return AjaxResult.error("合集名称不能重复!");
            }
            communityCollection.setUserId(SecurityUtils.getLoginUser().getUserId());
            communityCollection.setCreateBy(SecurityUtils.getLoginUser().getUserId());
            communityCollection.setCreateTime(DateUtils.parseDate(DateUtils.getTime()));
            communityCollection.setDelete(false);
            communityCollectionService.save(communityCollection);
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success(communityCollection);
    }


    /**
     * 更新合集信息
     *
     * @param communityCollection
     * @return
     */
    @ApiOperation("更新合集信息")
    @PostMapping("/updateCollection")
    @Transactional
    public AjaxResult updateCollection(@RequestBody CommunityCollection communityCollection) {
        if (Objects.isNull(communityCollection.getId())) {
            return AjaxResult.error("合集ID不能为空");
        }
        Long userId = SecurityUtils.getLoginUser().getUserId();
        try {
            communityCollection.setUpdateBy(userId);
            communityCollection.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
            communityCollectionMapper.updateById(communityCollection);
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success("文章更新成功!");
    }


    /**
     * 用户删除合集
     *
     * @param collectionId 合集id
     * @return 是否成功
     */
    @ApiOperation("用户删除合集")
    @DeleteMapping("/deleteCollection")
    @Transactional
    public AjaxResult deleteCollection(Long collectionId) {
        if (Objects.isNull(collectionId)) {
            return AjaxResult.error("参数异常!");
        }

        CommunityCollection collection;
        try {
            collection = communityCollectionMapper.selectOne(new QueryWrapper<CommunityCollection>()
                    .eq("id", collectionId)
                    .and(wrapper -> wrapper.ne("is_delete", true).or().isNull("is_delete")));
        } catch (Exception e) {
            return AjaxResult.error("查询失败，请稍后再试!");
        }

        if (collection == null) {
            return AjaxResult.error("合集不存在或已删除!");
        }

        Long userId = SecurityUtils.getLoginUser().getUserId();
        if (!collection.getCreateBy().equals(userId)) {
            return AjaxResult.error("合集只能本人删除!");
        }

        try {
            communityCollectionService.deleteCollectionById(collectionId);
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success("删除成功!");
    }


    /**
     * 合集内新增文章
     *
     * @param collectionArticle 合集内新增文章
     * @return 是否成功
     */
    @ApiOperation("合集内新增文章")
    @PostMapping("/addArticleToCollection")
    @Transactional
    public AjaxResult addArticleToCollection(@RequestBody CommunityCollectionArticle collectionArticle) {
        if (Objects.isNull(collectionArticle.getArticleId())) {
            return AjaxResult.error("文章参数异常!");
        }

        if (Objects.isNull(collectionArticle.getCollectionId())) {
            return AjaxResult.error("合集参数异常!");
        }

        /**
         * 1.查询当前文章是不是属于该用户的
         * 2.查询当前用户有没有这个合集
         * 3.情况1 如果文章在合集下,更新这个合集的删除状态，在插入到新的合集
         * 4.情况2 如果文章就在需要添加的合集下，则返回  该文章在合集已经存在
         * .如果没有在合集下，直接插入
         */
        Long collectionId = collectionArticle.getCollectionId();
        Long articleId = collectionArticle.getArticleId();
        Long userId = SecurityUtils.getLoginUser().getUserId();


        try {
            //查询当前文章是不是属于该用户的
            List<CommunityArticle> communityArticles = communityArticleMapper.selectList(new QueryWrapper<CommunityArticle>()
                    .eq("user_id", userId)
                    .eq("id", articleId)
                    .and((wrapper) -> {
                        wrapper.ne("is_delete", 1).or().isNull("is_delete");
                    }));

            if (communityArticles.isEmpty()) {
                return AjaxResult.success("文章ID不属于当前用户!");
            }

            //查询当前用户有没有这个合集
            List<CommunityCollection> communityCollections = communityCollectionMapper.selectList(new QueryWrapper<CommunityCollection>()
                    .eq("user_id", userId)
                    .eq("id", collectionId)
                    .and((wrapper) -> {
                        wrapper.ne("is_delete", 1).or().isNull("is_delete");
                    }));

            if (communityCollections.isEmpty()) {
                return AjaxResult.success("合集ID不属于当前登录用户!");
            }

            log.info("communityCollections: " + communityCollections);
            //根据创建人ID,合集ID,文章ID保持唯一性 查询表如果没有则插入,有了则更新ID
            List<CommunityCollectionArticle> communityCollectionArticles = communityCollectionArticleMapper.selectList(new QueryWrapper<CommunityCollectionArticle>()
                    .eq("create_by", userId)
                    .eq("article_id", articleId)
                    .and((wrapper) -> {
                        wrapper.ne("is_delete", 1).or().isNull("is_delete");
                    }));
            log.info("communityCollectionArticles: " + communityCollectionArticles);

            if (!communityCollectionArticles.isEmpty()) {
                for (CommunityCollectionArticle communityCollectionArticle : communityCollectionArticles) {
                    Long collectionId1 = communityCollectionArticle.getCollectionId();
                    // 如果 id 与 collectionId 相等，直接返回
                    if (collectionId1.equals(collectionId)) {
                        return AjaxResult.success("该文章在合集已经存在!");
                    } else {
                        // 如果不相等，执行删除操作
                        communityCollectionService.deleteArticleToCollection(collectionId1, articleId);
                    }
                }
            }
            collectionArticle.setCreateBy(userId);
            collectionArticle.setCreateTime(DateUtils.parseDate(DateUtils.getTime()));
            communityCollectionService.addArticleToCollection(collectionArticle);
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success("加入合集成功!");
    }

    /**
     * 合集内删除文章
     *
     * @param collectionId 合集id
     * @param articleId    文章id
     * @return 是否成功
     */
    @ApiOperation("合集内删除文章")
    @DeleteMapping("/deleteArticleInCollection")
    @Transactional
    public AjaxResult deleteArticleInCollection(Long collectionId, Long articleId) {
        if (Objects.isNull(collectionId) || Objects.isNull(articleId)) {
            return AjaxResult.error("参数异常!");
        }
        try {
            communityCollectionService.deleteArticleToCollection(collectionId, articleId);
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success("文章删除成功!");
    }

    /**
     * 查询合集下文章信息
     *
     * @param collectionId 合集id
     * @return 合集下文章信息
     */
    @ApiOperation("查询用户合集")
    @GetMapping("/getCollection")
    @Anonymous
    public AjaxResult getCollection(Long collectionId, Long searchType, @RequestParam(required = false) Long userId) {
        if (Objects.isNull(collectionId)) {
            return AjaxResult.error("合集不存在!");
        }

        if (Objects.isNull(userId)) {
            userId = SecurityUtils.getUserId();
        }

        List<CommunityCollectionArticleVo> collectionArticleVos = null;
        JSONArray jsonArray;
        try {
            jsonArray = communityCollectionService.selectArticleInfoInCollection(userId, collectionId, searchType);
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success(jsonArray);
    }

    /**
     * 搜索关键字
     *
     * @param content
     * @return 内容
     * fangqing
     */
    @ApiOperation("查询关键词")
    @GetMapping("/trendingTopic")
    //@Anonymous
    public AjaxResult TrendingTopic(@RequestParam(required = false) String content) {
        List<TrendingTop> trendingTops = null;
        try {
            trendingTops = communityArticleMapper.selectTrendingTopByContent(content);
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success("获取成功!", trendingTops);
    }

    /**
     * 订阅/取订合集
     *
     * @param collectionUser
     * @return 内容
     * fangqing
     */
    @ApiOperation("订阅/取订合集")
    @PostMapping("/subscribeCollection")
    @Transactional
    //@Anonymous
    public AjaxResult subscribeCollection(@RequestBody CommunityCollectionUser collectionUser) {
        if (Objects.isNull(collectionUser.getCollectionId())) {
            return AjaxResult.error("参数异常!");
        }

        CommunityCollectionUser communityCollectionUser = null;
        try {
            communityCollectionUser = communityCollectionUserService.subscribeCollection(collectionUser);

            if (communityCollectionUser == null) {
                return AjaxResult.success("不能关注自己创建的合集/传参有误");
            }
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success(communityCollectionUser);
    }

    /**
     * 获取个人中心信息
     */
    @ApiOperation("获取个人中心信息")
    @GetMapping("/userPrompt")
    public AjaxResult userPrompt(Long userId) {
        List<CommunityUserPromptVo> communityUserPromptVo = null;
        try {
            communityUserPromptVo = communityArticleService.selectCommunityUserPrompt(userId);
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success(communityUserPromptVo);
    }

    @ApiOperation("个人未加入集合文章")
    @GetMapping("/userNoCo")
    @Anonymous
    public AjaxResult userNoCo(@RequestParam(required = false) Long userId) {
        int pageNum = Convert.toInt(ServletUtils.getParameter("pageNum"), 1);
        int pageSize = Convert.toInt(ServletUtils.getParameter("pageSize"), 10);
        int searchType = Convert.toInt(ServletUtils.getParameter("searchType"), 0);
        List<CommunityArticleVo> communityArticleVos = null;
        if (userId == null) {
            userId = SecurityUtils.getUserId();
        }
        try {
            communityArticleVos = communityArticleService.selectUserNoCo(userId, pageNum, pageSize, searchType);
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success(communityArticleVos);
    }

    @ApiOperation("个人喜欢文章")
    @GetMapping("/userLike")
    @Anonymous
    public AjaxResult userLike(@RequestParam(required = false) Long userId) {
        int pageNum = Convert.toInt(ServletUtils.getParameter("pageNum"), 1);
        int pageSize = Convert.toInt(ServletUtils.getParameter("pageSize"), 10);
        int searchType = Convert.toInt(ServletUtils.getParameter("searchType"), 1);
        List<CommunityArticleVo> communityArticleVos = null;
        if (userId == null) {
            userId = SecurityUtils.getUserId();
        }
        try {
            communityArticleVos = communityArticleService.selectUserLike(userId, pageNum, pageSize, searchType);

            if (communityArticleVos != null) {
                for (CommunityArticleVo communityArticleVo : communityArticleVos) {
                    //判断当前查询用户有没有喜欢该文章
                    CommunityLike like = communityLikeMapper.selectOne(
                            new QueryWrapper<CommunityLike>()
                                    .eq("user_id", SecurityUtils.getUserId())
                                    .eq("article_id", communityArticleVo.getId()));

                    communityArticleVo.setLike(!Objects.isNull(like));

                    //判断是否已推荐
                    CommunityArticleRecommend recommend = recommendMapper.selectOne(new QueryWrapper<CommunityArticleRecommend>()
                            .eq("user_id", SecurityUtils.getUserId())
                            .eq("article_id", communityArticleVo.getId()).and((wrapper) -> {
                                wrapper.ne("is_delete", true).or().isNull("is_delete");
                            }));
                    communityArticleVo.setRecommend(!Objects.isNull(recommend));
                }
            }
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success(communityArticleVos);
    }

    @ApiOperation("个人收藏文章")
    @GetMapping("/userCollect")
    @Anonymous
    public AjaxResult userCollect(@RequestParam(required = false) Long userId) {
        int pageNum = Convert.toInt(ServletUtils.getParameter("pageNum"), 1);
        int pageSize = Convert.toInt(ServletUtils.getParameter("pageSize"), 10);
        int searchType = Convert.toInt(ServletUtils.getParameter("searchType"), 1);

        List<CommunityArticleVo> communityArticleVos = null;
        try {
            if (userId == null) {
                userId = SecurityUtils.getUserId();
            }
            communityArticleVos = communityArticleService.selectUserCollect(userId, pageNum, pageSize, searchType);

            if (communityArticleVos != null) {
                for (CommunityArticleVo communityArticleVo : communityArticleVos) {
                    //判断当前查询用户有没有喜欢该文章
                    CommunityLike like = communityLikeMapper.selectOne(
                            new QueryWrapper<CommunityLike>()
                                    .eq("user_id", SecurityUtils.getUserId())
                                    .eq("article_id", communityArticleVo.getId()));

                    communityArticleVo.setLike(!Objects.isNull(like));

                    //判断是否已推荐
                    CommunityArticleRecommend recommend = recommendMapper.selectOne(new QueryWrapper<CommunityArticleRecommend>()
                            .eq("user_id", SecurityUtils.getUserId())
                            .eq("article_id", communityArticleVo.getId()).and((wrapper) -> {
                                wrapper.ne("is_delete", true).or().isNull("is_delete");
                            }));
                    communityArticleVo.setRecommend(!Objects.isNull(recommend));
                }
            }
        } catch (Exception e) {
            log.info(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success(communityArticleVos);
    }

    @ApiOperation("个人陪伴信息")
    @GetMapping("/userAccompany")
    @Anonymous
    public AjaxResult userAccompany(Long userId) {

        if (Objects.isNull(userId)) {
            userId = SecurityUtils.getUserId();
        }

        CommunityAccompany communityAccompany = null;
        try {
            communityAccompany = communityAccompanyService.selectCommunityAccompanyById(userId);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return AjaxResult.success(communityAccompany);
    }

    /**
     * 插入at信息
     */
    @ApiOperation("插入at信息")
    @PostMapping("/articleAt")
    @Anonymous
    public AjaxResult articleAt(@RequestBody CommunityArticleAt communityArticleAt) {
        if (Objects.isNull(communityArticleAt.getPeerId())) {
            return AjaxResult.error("@人ID不能为空");
        }
        Long userId = SecurityUtils.getUserId();
        try {
            communityArticleAt.setCreateBy(userId);
            communityArticleAt.setCreateTime(DateUtils.parseDate(DateUtils.getTime()));
            communityArticleAtMapper.insert(communityArticleAt);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return AjaxResult.success(communityArticleAt);
    }

    /**
     * 置顶文章
     */
    @ApiOperation("置顶文章")
    @PostMapping("/articleTop")
    @Anonymous
    public AjaxResult articleTop(@RequestBody CommunityArticle communityArticle) {
        if (Objects.isNull(communityArticle.getId())) {
            return AjaxResult.error("文章ID不能为空");
        }
        //先拿前端传的ID
        Long userId = communityArticle.getUserId();
        if (Objects.isNull(communityArticle.getUserId())) {
            userId = SecurityUtils.getUserId();
        }
        String msg = "";
        try {
            CommunityArticle toparticle = communityArticleMapper.selectOne(new QueryWrapper<CommunityArticle>()
                    .eq("user_id", userId)
                    .eq("id", communityArticle.getId())
                    .and((wrapper) -> {
                        wrapper.ne("is_delete", 1).or().isNull("is_delete");
                    }));
            //如果没有找到数据置顶该文章 说明不是当前用户
            if (Objects.isNull(toparticle) || Objects.isNull(toparticle.getId())) {
                return AjaxResult.success("文章不属于当前用户,无法置顶");
            }
            Boolean isTop = toparticle.getIsTop();
            if (!isTop) {
                communityArticleService.update(new UpdateWrapper<CommunityArticle>()
                        .set("is_top", true)
                        .set("top_time", DateUtils.parseDate(DateUtils.getTime()))
                        .eq("id", communityArticle.getId()));
                msg = "已置顶";
            } else {
                communityArticleService.update(new UpdateWrapper<CommunityArticle>()
                        .set("is_top", false)
                        .set("top_time", null)
                        .eq("id", communityArticle.getId()));
                msg = "已取消置顶";
            }
            //每个人只能有三条置顶数据，如果有第四条或者第五条数据,则按时间排序找到离当前时间最近的第三条数据更新掉小于当前时间的所有数据为false
            List<CommunityArticle> communityArticles = communityArticleMapper.selectList(new QueryWrapper<CommunityArticle>()
                    .eq("user_id", userId)
                    .eq("is_top", 1)
                    .and((wrapper) -> {
                        wrapper.ne("is_delete", 1).or().isNull("is_delete");
                    }));
            if (communityArticles.size() > 3) {
                //大于三条则 排序
                communityArticles = communityArticles.stream()
                        .sorted(Comparator.comparing(CommunityArticle::getTopTime).reversed()) // 降序排列（最新文章排在前面）
                        .limit(3)
                        .collect(Collectors.toList());

                //最早文章的 is_top 更新为 false
                UpdateWrapper<CommunityArticle> updateWrapper = new UpdateWrapper<>();
                updateWrapper.eq("user_id", userId)
                        .eq("is_top", 1)
                        .notIn("id", communityArticles.stream().map(CommunityArticle::getId).collect(Collectors.toList()))
                        .set("top_time", null)
                        .set("is_top", false); // 将 is_top 设置为 false

                communityArticleMapper.update(null, updateWrapper);
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
        return AjaxResult.success(msg);
    }

}
