package com.ruoyi.generator.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.annotation.Anonymous;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.core.text.Convert;
import com.ruoyi.common.exception.user.ProjectException;
import com.ruoyi.common.utils.*;
import com.ruoyi.common.utils.ip.AddressUtils;
import com.ruoyi.generator.domain.Community.*;
import com.ruoyi.generator.mapper.community.*;
import com.ruoyi.generator.service.*;
import com.ruoyi.generator.vo.*;
import com.ruoyi.system.mapper.SysUserMapper;
import com.ruoyi.system.service.impl.SysUserServiceImpl;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.ExecutionException;
import java.util.stream.Collectors;

/**
 * 评论接口
 *
 * @author fangzhen
 */
@Api(tags = "评论接口")
@RestController
@RequestMapping("/community/comment")
public class CommunityCommentController extends BaseController {

    @Autowired
    private ICommunityCommentReplyService communityCommentReplyService;

    @Autowired
    private ICommunityArticleCommentService communityArticleCommentService;

    @Autowired
    private ICommunityLikeService communityLikeService;

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private CommunityCommentLikeMapper communityCommentLikeMapper;

    @Autowired
    private SysUserServiceImpl sysUserServiceImpl;

    @Autowired
    private SysUserServiceImpl sysUserService;

    @Autowired
    private ICommunityArticleService communityArticleService;

    @Autowired
    private CommunityArticleMapper communityArticleMapper;

    @Autowired
    private CommunityArticleCommentMapper communityArticleCommentMapper;

    @Autowired
    private CommunityCommentReplyMapper communityCommentReplyMapper;

    @Autowired
    private CommunityArticleCollectMapper communityArticleCollectMapper;

    @Autowired
    private CommunityArticleCollectServiceImpl articleCollectService;

    @Autowired
    private CommunityArticleRecommendMapper communityArticleRecommendMapper;

    @Autowired
    private CommunityArticleRecommendServiceImpl communityArticleRecommendServiceImpl;

    @Resource
    private CommunityCommentLikeServiceImpl commentLikeService;

    @Resource
    private CommunityUserLikeServiceImpl userLikeService;

    @Autowired
    private CommunityUserLikeMapper communityUserLikeMapper;

    @Autowired
    private RedisCache redisCache;
    @Autowired
    private CommunityCommentRaffleMapper communityCommentRaffleMapper;

    @Autowired
    private CommunityArticleAtServiceImpl communityArticleAtService;

    @Autowired
    private CommunityReportController communityReportController;

    @Autowired
    private CommunityCommentController communityCommentController;
    /**
     * 获取文章评论
     */
    @ApiOperation("获取文章评论")
    @GetMapping()
    @Transactional
    @Anonymous
    public AjaxResult comment(String articleId,
                              int currentPage,
                              int limit,
                              @RequestParam(required = false) Long userId,
                              String commentId) {
        if (Objects.isNull(articleId)) {
            return AjaxResult.error("参数异常!");
        }
        List<CommunityArticleCommentVo> commentVos = new ArrayList<>();

        if (userId == null) {
            userId = SecurityUtils.getUserId();
        }

        //设置评论回复数量
        int total =  communityArticleCommentMapper.queryCommentCount(Long.valueOf(articleId));


        try {
            Page<CommunityArticleComment> page = new Page<>(currentPage, limit);
            QueryWrapper<CommunityArticleComment> queryWrapper = new QueryWrapper<CommunityArticleComment>()
                    .eq("article_id", articleId)
                    .and((wrapper) -> {
                        wrapper.ne("is_delete", true).or().isNull("is_delete");
                    }).orderByDesc("create_time");

            // 只有在 commentId 不为空时才添加 id 的条件
            if (commentId != null && !commentId.isEmpty()) {
                queryWrapper.eq("id", commentId);
            }

            List<CommunityArticleComment> records = communityArticleCommentService.page(page, queryWrapper).getRecords();

            CommunityArticleCommentVo commentVo = null;

            for (CommunityArticleComment record : records) {
                if (record.getAddress() == null) {
                    record.setAddress("未知");
                }

                commentVo = new CommunityArticleCommentVo();
                BeanUtils.copyProperties(record, commentVo);
                commentVos.add(commentVo);
            }


            List<Long> commentIds = new ArrayList<>();
            for (CommunityArticleCommentVo vo : commentVos) {
                Long commentUserId = vo.getUserId();
                //获取评论的用户信息
                SysUser sysUser = sysUserMapper.selectUserById(commentUserId);
                vo.setNickName(sysUser.getNickName());
                vo.setAvatar(sysUser.getAvatar());
                //当前登录用户是否已点赞
                List<CommunityCommentLike> commentLikes = communityCommentLikeMapper.selectList(new QueryWrapper<CommunityCommentLike>()
                        .eq("comment_id", vo.getId())
                        .eq("user_id", userId));
                vo.setCommentLike(!commentLikes.isEmpty());
                //该评论的点赞数量
                vo.setCommentLikeCount(communityCommentLikeMapper.selectList(new QueryWrapper<CommunityCommentLike>()
                                .eq("comment_id", vo.getId()))
                        .size());
                //设置首条评论
                List<CommunityCommentReply> replies = communityCommentReplyService.list(new QueryWrapper<CommunityCommentReply>()
                        .eq("comment_id", vo.getId())
                        .and((wrapper) -> {
                            wrapper.ne("is_delete", true).or().isNull("is_delete");
                        }).orderByAsc("create_time"));

                //如果评价用户id与当前登录用户id相符,则标记为已读
                if (vo.getUserId().equals(userId) && !vo.isRead()) {
                    commentIds.add(vo.getId());
                }

                CommunityCommentReplyVo replyVo = null;
                List<CommunityCommentReplyVo> replyVos = new ArrayList<>();
                for (CommunityCommentReply reply : replies) {
                    replyVo = new CommunityCommentReplyVo();
                    BeanUtils.copyProperties(reply, replyVo);
                    replyVo.setNickName(sysUserService.selectUserById(replyVo.getUserId()).getNickName());
                    replyVo.setReply(true);
                    replyVo.setAvatar(sysUserService.selectUserById(replyVo.getUserId()).getAvatar());
                    replyVos.add(replyVo);
                    //设置点赞数量和当前登录用户是否已点赞
                    List<CommunityCommentLike> commentLikeList = communityCommentLikeMapper.selectList(new QueryWrapper<CommunityCommentLike>()
                            .eq("reply_id", replyVo.getId()));
                    replyVo.setLikeCount(commentLikeList.size());
                    replyVo.setLike(commentLikeList.stream().anyMatch(item -> SecurityUtils.getUserId().equals(item.getUserId())));
                    break;
                }
                vo.setFirstReply(replyVos);
                //设置还剩多少评论
                vo.setReplyCount(replies.isEmpty() ? 0 : replies.size() - 1);
                vo.setReply(false);
            }

            //更新为已读
            /*if (!commentIds.isEmpty()) {
                communityArticleCommentService.update(new UpdateWrapper<CommunityArticleComment>().set("is_read", true).in("id", commentIds));
            }*/
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.successByCounts(commentVos,total);
    }

    /**
     * 获取评论回复
     */
    @ApiOperation("获取评论回复")
    @GetMapping("/reply")
    @Transactional
    //@Anonymous
    public AjaxResult reply(String commentId, int currentPage, int limit) {
        if (Objects.isNull(commentId)) {
            return AjaxResult.error("参数异常!");
        }

        List<CommunityCommentReplyVo> replyVos = new ArrayList<>();
        try {
            Long userId = SecurityUtils.getUserId();
            Page<CommunityCommentReply> page = new Page<>(currentPage, limit);
            List<CommunityCommentReply> replies = communityCommentReplyService.page(page, new QueryWrapper<CommunityCommentReply>()
                    .eq("comment_id", commentId)
                    .and((wrapper) -> {
                        wrapper.ne("is_delete", true)
                                .or()
                                .isNull("is_delete");
                    }).orderByAsc("create_time")).getRecords();

            CommunityCommentReplyVo replyVo = null;
            List<Long> replyIds = new ArrayList<>();
            for (final CommunityCommentReply reply : replies) {
                replyVo = new CommunityCommentReplyVo();
                if (reply.getAddress() == null) {
                    reply.setAddress("未知");
                }

                //标记当前登录的用户被回复的信息
                if (reply.getUserId().equals(userId)) {
                    List<CommunityCommentReply> communityCommentReplies = replies.stream()
                            .filter(item -> item.getParentReplyId().equals(reply.getId()))
                            .collect(Collectors.toList());
                    for (CommunityCommentReply communityCommentReply : communityCommentReplies) {
                        replyIds.add(communityCommentReply.getId());
                    }
                }

                SysUser sysUser = sysUserService.selectUserById(reply.getUserId());
                final Long parentReplyId = reply.getParentReplyId();
                BeanUtils.copyProperties(reply, replyVo);
                if (Objects.nonNull(parentReplyId) && !commentId.equalsIgnoreCase(Long.toString(parentReplyId))) {
                    List<CommunityCommentReply> communityCommentReplies = replies.stream()
                            .filter(item -> item.getId().equals(parentReplyId))
                            .collect(Collectors.toList());
                    CommunityCommentReply communityCommentReply = communityCommentReplies.get(0);
                    SysUser sysUser1 = sysUserService.selectUserById(communityCommentReply.getUserId());
                    replyVo.setParentNickName(sysUser1.getNickName());
                    replyVo.setParentUserId(sysUser1.getUserId());
                    replyVo.setParentAvatar(sysUser1.getAvatar());
                }
                replyVo.setNickName(sysUser.getNickName());
                replyVo.setAvatar(sysUser.getAvatar());
                replyVo.setReply(true);
                //设置点赞数量和当前登录用户是否已点赞
                List<CommunityCommentLike> commentLikes = communityCommentLikeMapper.selectList(new QueryWrapper<CommunityCommentLike>()
                        .eq("reply_id", replyVo.getId()));
                replyVo.setLikeCount(commentLikes.size());
                replyVo.setLike(commentLikes.stream().anyMatch(item -> SecurityUtils.getUserId().equals(item.getUserId())));
                replyVos.add(replyVo);
            }

            //更新回复的id为已读
            if (!replyIds.isEmpty()) {
                communityCommentReplyService.update(new UpdateWrapper<CommunityCommentReply>()
                        .set("is_read", true)
                        .in("id", replyIds));
            }
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success(replyVos);
    }


    /**
     * 对文章评论进行回复
     */
    @ApiOperation("对文章评论进行回复")
    @PostMapping("/reply")
    @Transactional
    //@Anonymous
    public AjaxResult reply(@RequestBody CommunityCommentReply communityCommentReply) throws ParseException {
        if (Objects.isNull(communityCommentReply.getCommentId()) || Objects.isNull(communityCommentReply.getContent())) {
            return AjaxResult.error("参数异常!");
        }


        Long userId = SecurityUtils.getUserId();
        try {
            //判断当前用户有没有被禁言
            SysUser sysUsers = sysUserMapper.selectUserById(userId);
            if (sysUsers != null){
                String userState = sysUsers.getUserState();
                Date reportTime = sysUsers.getReportTime();
                if ("2".equals(userState) && reportTime != null) {
                    communityReportController.adminReportPunish(userId);
                    // 获取当前时间
                    Date currentTime = new Date();
                    // 检查 reportTime 小于当前时间
                    if (currentTime.before(reportTime)) {
                        // 返回封号的提示
                        return   AjaxResult.success("当前用户已被禁言!");
                    }
                }
            }

            //过滤敏感词
            boolean isSensitive = SensitiveWordUtil.containsSensitiveWord(communityCommentReply.getContent());
            if (isSensitive) {
                return AjaxResult.error(MessageUtils.message("article.user.comment"));
            }

            Boolean aBoolean = communityArticleService.checkCommentPermission(userId, communityCommentReply.getCommentId(), null);
            if (!aBoolean) {  // 当 aBoolean 为 false 时，进入这个条件
                return AjaxResult.success("您没有权限评论此文章，因为作者只允许关注的人评论!");
            }

            communityCommentReply.setUserId(userId);
            communityCommentReply.setDelete(false);
            communityCommentReply.setCreateBy(userId);
            communityCommentReply.setCreateTime(DateUtils.parseDate(DateUtils.getTime()));
            String address = null;
            try {
                address = AddressUtils.getAddress(null);
            } catch (ExecutionException e) {
                throw new RuntimeException(e);
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }
            communityCommentReply.setAddress(address);
            communityCommentReplyService.save(communityCommentReply);

            //拿到需要At人的ID
            List<Long> atUserIds = communityCommentReply.getAtUserIds();
            if (atUserIds != null && !atUserIds.isEmpty()) {
                List<CommunityArticleAt> communityArticleAts = new ArrayList<>();
                CommunityArticleAt communityArticleAt = null;
                for (Long atUserId : atUserIds) {
                    communityArticleAt = new CommunityArticleAt();
                    communityArticleAt.setType(2L);

                    //获取文章ID
                    CommunityArticleComment communityArticleComment = communityArticleCommentMapper.selectOne(new QueryWrapper<CommunityArticleComment>()
                            .eq("id", communityCommentReply.getCommentId())
                            .and((wrapper) -> {
                                wrapper.ne("is_delete", true).or().isNull("is_delete");
                            }));
                    communityArticleAt.setArticleId(communityArticleComment.getArticleId());
                    communityArticleAt.setCommentId(communityCommentReply.getCommentId());
                    communityArticleAt.setReplyId(communityCommentReply.getId());
                    communityArticleAt.setUserId(SecurityUtils.getLoginUser().getUserId());
                    communityArticleAt.setPeerId(atUserId);
                    communityArticleAt.setCreateBy(SecurityUtils.getLoginUser().getUserId());
                    communityArticleAt.setCreateTime(DateUtils.parseDate(DateUtils.getTime()));
                    communityArticleAts.add(communityArticleAt);
                }
                communityArticleAtService.saveBatch(communityArticleAts);
            }

        } catch (Exception e) {
           // e.printStackTrace();
            System.out.println(e.getMessage());
            throw new ProjectException();
        }


        return AjaxResult.success(communityCommentReply);
    }

    /**
     * 删除回复
     */
    @ApiOperation("删除回复")
    @DeleteMapping("/deleteReply")
    @Transactional
    //@Anonymous
    public AjaxResult deleteReply(String replyId) {
        if (Objects.isNull(replyId)) {
            return AjaxResult.error("参数异常!");
        }
        try {
            CommunityCommentReply communityCommentReply = communityCommentReplyService.getOne(new QueryWrapper<CommunityCommentReply>().eq("id", replyId).and((wrapper) -> {
                wrapper.ne("is_delete", true).or().isNull("is_delete");
            }));
            if (Objects.isNull(communityCommentReply)) {
                return AjaxResult.error("未找到该回复!");
            }
            Long userId = SecurityUtils.getUserId();
            communityCommentReply.setDelete(true);
            communityCommentReply.setUpdateBy(userId);
            communityCommentReply.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
            communityCommentReplyService.updateById(communityCommentReply);
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success("删除回复成功!");
    }


    /**
     * @param articleId 文章id
     * @return 抽奖结果
     */
    @ApiOperation("评论抽奖")
    @GetMapping("/raffleComment")
    @Transactional
    //@Anonymous
    public AjaxResult raffleComment(Long articleId) {
        if (Objects.isNull(articleId)) {
            return AjaxResult.error("参数异常!");
        }

        CommunityArticle communityArticle = communityArticleMapper.selectOne(new QueryWrapper<CommunityArticle>()
                .eq("id", articleId)
                .eq("create_by", SecurityUtils.getUserId())
                .and((wrapper) -> {
                    wrapper.ne("is_delete", true).or().isNull("is_delete");
                }));
        if (communityArticle == null) {
            return AjaxResult.error("非本人无法操作抽奖按钮!");
        }

        CommunityCommentRaffleVo communityCommentRaffleVos = null;
        try {
            communityCommentRaffleVos = communityArticleService.selectCommentRaffle(articleId);
//            redisCache.setCacheList(CacheConstants.COMMUNITY_COMMENT_RAFFLE, communityCommentRaffleMapper.selectList(new QueryWrapper<CommunityCommentRaffle>().eq("is_delete", false)));
//            redisCache.expire(CacheConstants.COMMUNITY_COMMENT_RAFFLE, 24, TimeUnit.HOURS);
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success(communityCommentRaffleVos);
    }


    @ApiOperation("查询评论")
    @GetMapping("/queryComment")
    @Transactional
    //@Anonymous
    public AjaxResult queryComment(Long articleId, String comment) {
        int pageNum = Convert.toInt(ServletUtils.getParameter("pageNum"), 1);
        int pageSize = Convert.toInt(ServletUtils.getParameter("pageSize"), 10);
        if (Objects.isNull(articleId)) {
            return AjaxResult.error("参数异常!");
        }
        Long userId = SecurityUtils.getUserId();
        List<CommunitySearchCommentVo> commentVos = new ArrayList<>();
        try {
            int offset = (pageNum - 1) * pageSize;
            //获取一二级评论
            List<CommunityArticleComment> communityArticleComments = communityArticleCommentMapper.queryComment(articleId, comment, offset, pageSize);
            //一二级评论合集Vo
            CommunitySearchCommentVo commentVo = null;
            for (CommunityArticleComment communityArticleComment : communityArticleComments) {
                commentVo = new CommunitySearchCommentVo();
                BeanUtils.copyProperties(communityArticleComment, commentVo);
                commentVos.add(commentVo);
            }
            for (CommunitySearchCommentVo vo : commentVos) {
                //拼接头像 信息
                vo.setAvatar(sysUserService.selectUserById(vo.getUserId()).getAvatar());
                vo.setNickName(sysUserService.selectUserById(vo.getUserId()).getNickName());
                vo.setCommentLikeCount(communityCommentLikeMapper.selectList(new QueryWrapper<CommunityCommentLike>().eq("comment_id", vo.getId())).size());
                //设置当前用户是否点赞
                List<CommunityCommentLike> commentLikes = communityCommentLikeMapper.selectList(new QueryWrapper<CommunityCommentLike>()
                        .eq("comment_id", vo.getId())
                        .eq("user_id", userId));
                vo.setCommentLike(!commentLikes.isEmpty());
                //如果是二级评论则执行这个接口
                CommunityCommentReply communityCommentReply = communityCommentReplyMapper.selectOne(new QueryWrapper<CommunityCommentReply>()
                        .eq("id", vo.getId())
                        .and((wrapper) -> {
                            wrapper.ne("is_delete", true).or().isNull("is_delete");
                        }));

                //如果二级评论不为空执行
                if (!Objects.isNull(communityCommentReply)) {
                    //点赞数量
                    List<CommunityCommentLike> commentLikeList = communityCommentLikeMapper.selectList(new QueryWrapper<CommunityCommentLike>()
                            .eq("reply_id", communityCommentReply.getId()));
                    vo.setCommentLikeCount(commentLikeList.size());

                    //二级评论是否点赞
                    List<CommunityCommentLike> commentLikesReply = communityCommentLikeMapper.selectList(new QueryWrapper<CommunityCommentLike>()
                            .eq("reply_id", vo.getId())
                            .eq("user_id", userId));
                    vo.setCommentLike(!commentLikesReply.isEmpty());

                    //找到一级评论获取 父评论 用户头像和ID
                    CommunityArticleComment communityArticleComment = communityArticleCommentMapper.selectOne(new QueryWrapper<CommunityArticleComment>()
                            .eq("id", communityCommentReply.getCommentId())
                            .and((wrapper) -> {
                                wrapper.ne("is_delete", true).or().isNull("is_delete");
                            }));
                    SysUser sysUser1 = sysUserService.selectUserById(communityArticleComment.getUserId());
                    vo.setParentNickName(sysUser1.getNickName());
                    vo.setParentUserId(sysUser1.getUserId());
                    vo.setParentAvatar(sysUser1.getAvatar());
                }
            }
        } catch (Exception e) {
            //e.printStackTrace();
            System.out.println(e.getMessage());
            throw new ProjectException();
        }
        System.out.println(commentVos);
        return AjaxResult.success(commentVos);
    }
    /**
     * 获取当前登录用户个人中心信息数量
     *
     * @return 个人中心信息数量
     */
    @GetMapping("/getMyMessageNum")
    @ApiOperation("获取当前登录用户个人中心信息数量")
    public AjaxResult getMyMessage() {
        Long userId = SecurityUtils.getUserId();
        int pageNum = Convert.toInt(ServletUtils.getParameter("pageNum"), 1);
        int pageSize = Convert.toInt(ServletUtils.getParameter("pageSize"), 1000000000);
        int searchType = Convert.toInt(ServletUtils.getParameter("searchType"), 1);
        int offset = (pageNum - 1) * pageSize;

        MyMessageNumVo myMessageNumVo = new MyMessageNumVo();

        //获取当前登录用户的评论和回复 数量
        List<CommentVo> commentInfoByUserId = communityArticleCommentService.getCommentInfoByUserId(userId, offset, pageSize, searchType);
        long unreadCommentCount = commentInfoByUserId.stream()
                .filter(comment -> !comment.isRead())
                .count();

        myMessageNumVo.setComments((int) unreadCommentCount);

        //获取当前登录用户被推荐和收藏的信息
        List<RecommendAndCollectVo> recommendAndCollectVos = communityArticleCollectMapper.selectRecommendAndCollectByUserId(userId, offset, pageSize, searchType);
        long unreadRecommendAndCollect = recommendAndCollectVos.stream()
                .filter(recommendAndCollect -> !recommendAndCollect.isRead())
                .count();
        myMessageNumVo.setLikeAndCollect((int) unreadRecommendAndCollect);

        //获取当前登录用户新增关注列表
        List<LikeVo> likeVos = communityUserLikeMapper.selectLikeByUserId(userId, offset, pageSize, searchType);
        long unreadlike = likeVos.stream()
                .filter(like -> !like.isRead())
                .count();
        myMessageNumVo.setLikeList((int) unreadlike);


        return success(myMessageNumVo);


    }
    /**
     * 获取当前登录用户的评论和回复
     *
     * @return 评论和回复
     */
    @GetMapping("/getComments")
    @ApiOperation("获取当前登录用户的评论和回复")
    public AjaxResult getCommentInfoByUserId() {
        Long userId = SecurityUtils.getUserId();
        int pageNum = Convert.toInt(ServletUtils.getParameter("pageNum"), 1);
        int pageSize = Convert.toInt(ServletUtils.getParameter("pageSize"), 10);
        int searchType = Convert.toInt(ServletUtils.getParameter("searchType"), 1);
        int offset = (pageNum - 1) * pageSize;

        List<CommentVo> commentInfoByUserId  = new ArrayList<>();
        List<CommentVo> pagedList  = new ArrayList<>();
        try {
            commentInfoByUserId = communityArticleCommentService.getCommentInfoByUserId2(userId,searchType);
            for (CommentVo commentVo : commentInfoByUserId) {
                String type = String.valueOf(commentVo.getType());
                String id = String.valueOf(commentVo.getId());
                if (!commentVo.isRead()){
                    communityCommentController.updateMsgRead(type,id);
                }
            }

            int toIndex = Math.min(offset + pageSize, commentInfoByUserId.size());
            // 检查是否越界，防止异常
            if (offset >= commentInfoByUserId.size() || offset < 0) {
                return success(new ArrayList<>()); // 返回空列表，避免异常
            }
            pagedList = commentInfoByUserId.subList(offset, toIndex);

        }catch (Exception e) {
            //e.printStackTrace();
            System.out.println(e.getMessage());
            throw new ProjectException();
        }
        return success(pagedList);
    }

    /**
     * 获取当前登录用户被推荐和收藏的信息
     *
     * @return 推荐和收藏
     */
    @GetMapping("/getLikeAndCollect")
    @ApiOperation("获取当前登录用户被推荐和收藏的信息")
    public AjaxResult getLikeAndCollect() {
        Long userId = SecurityUtils.getUserId();
        int pageNum = Convert.toInt(ServletUtils.getParameter("pageNum"), 1);
        int pageSize = Convert.toInt(ServletUtils.getParameter("pageSize"), 10);
        int searchType = Convert.toInt(ServletUtils.getParameter("searchType"), 1);
        int offset = (pageNum - 1) * pageSize;

        List<RecommendAndCollectVo> recommendAndCollectVos  = new ArrayList<>();
        List<RecommendAndCollectVo> pagedList  = new ArrayList<>();
        try {
            recommendAndCollectVos= communityArticleCollectMapper.selectRecommendAndCollectByUserId2(userId, searchType);

            for (RecommendAndCollectVo recommendAndCollectVo : recommendAndCollectVos) {
                String type = String.valueOf(recommendAndCollectVo.getType());
                String id = String.valueOf(recommendAndCollectVo.getId());
                if (!recommendAndCollectVo.isRead()){
                    communityCommentController.updateMsgRead(type,id);
                }
            }
            int toIndex = Math.min(offset + pageSize, recommendAndCollectVos.size());
            // 检查是否越界，防止异常
            if (offset >= recommendAndCollectVos.size() || offset < 0) {
                return success(new ArrayList<>()); // 返回空列表，避免异常
            }
            pagedList = recommendAndCollectVos.subList(offset, toIndex);

        }catch (Exception e) {
            //e.printStackTrace();
            System.out.println(e.getMessage());
            throw new ProjectException();
        }
        return success(pagedList);
    }

    /**
     * 获取当前登录用户新增关注列表
     *
     * @return 获取当前登录用户新增关注列表
     */
    @GetMapping("/getLikeList")
    @ApiOperation("获取当前登录用户新增关注列表")
    public AjaxResult getLikeList() {
        Long userId = SecurityUtils.getUserId();
        int pageNum = Convert.toInt(ServletUtils.getParameter("pageNum"), 1);
        int pageSize = Convert.toInt(ServletUtils.getParameter("pageSize"), 10);
        int searchType = Convert.toInt(ServletUtils.getParameter("searchType"), 1);
        int offset = (pageNum - 1) * pageSize;
        List<LikeVo> likeVos = new ArrayList<>();
        try {
            //调用这个接口 刷新已读状态
            likeVos = communityUserLikeMapper.selectLikeByUserId(userId, offset, pageSize, searchType);


            userLikeService.update(new UpdateWrapper<CommunityUserLike>()
                    .set("is_read", true)
                    .set("update_time", DateUtils.parseDate(DateUtils.getTime()))
                    .set("update_by", SecurityUtils.getUserId())
                    .eq("like_user_id",userId)
                    .eq("is_read", false));


        }catch (Exception e) {
            //e.printStackTrace();
            System.out.println(e.getMessage());
            throw new ProjectException();
        }
        return success(likeVos);
    }

    /**
     * 用户点击通知后,更新通知消息已读
     *
     * @return 更新结果
     */
    @PutMapping("/updateMsgRead")
    @ApiOperation("用户点击通知后,更新通知消息已读")
    public AjaxResult updateMsgRead(String type, String id) {
        switch (type) {
            case "0":
                //文章推荐
                communityArticleRecommendServiceImpl.update(new UpdateWrapper<CommunityArticleRecommend>()
                        .set("is_read", true)
                        .set("update_time", DateUtils.parseDate(DateUtils.getTime()))
                        .set("update_by", SecurityUtils.getUserId())
                        .eq("id", id));
                break;
            case "1":
                //文章收藏
                articleCollectService.update(new UpdateWrapper<CommunityArticleCollect>()
                        .set("is_read", true)
                        .set("update_time", DateUtils.parseDate(DateUtils.getTime()))
                        .set("update_by", SecurityUtils.getUserId())
                        .eq("id", id));
                break;
            case "2":
                //评论点赞
                commentLikeService.update(new UpdateWrapper<CommunityCommentLike>()
                        .set("is_read", true)
                        .set("update_time", DateUtils.parseDate(DateUtils.getTime()))
                        .set("update_by", SecurityUtils.getUserId())
                        .eq("id", id));
                break;
            case "3":
                //回复点赞
                /*communityCommentReplyService.update(new UpdateWrapper<CommunityCommentReply>()
                        .set("is_read", true)
                        .set("update_time", DateUtils.parseDate(DateUtils.getTime()))
                        .set("update_by", SecurityUtils.getUserId())
                        .eq("id", id));*/
                commentLikeService.update(new UpdateWrapper<CommunityCommentLike>()
                        .set("is_read", true)
                        .set("update_time", DateUtils.parseDate(DateUtils.getTime()))
                        .set("update_by", SecurityUtils.getUserId())
                        .eq("id", id));
                break;
            case "4":
                //新增关注已读
                userLikeService.update(new UpdateWrapper<CommunityUserLike>()
                        .set("is_read", true)
                        .set("update_time", DateUtils.parseDate(DateUtils.getTime()))
                        .set("update_by", SecurityUtils.getUserId())
                        .eq("id", id));
                break;
            case "5":
                //文章@
                communityArticleAtService.update(new UpdateWrapper<CommunityArticleAt>()
                        .set("is_read", true)
                        .set("update_time", DateUtils.parseDate(DateUtils.getTime()))
                        .set("update_by", SecurityUtils.getUserId())
                        .eq("id", id)
                        .eq("type",0));
                break;
            case "6":
                //文章评论@
                communityArticleAtService.update(new UpdateWrapper<CommunityArticleAt>()
                        .set("is_read", true)
                        .set("update_time", DateUtils.parseDate(DateUtils.getTime()))
                        .set("update_by", SecurityUtils.getUserId())
                        .eq("id", id)
                        .eq("type",1));
                break;
            case "7":
                //文章回复@
                communityArticleAtService.update(new UpdateWrapper<CommunityArticleAt>()
                        .set("is_read", true)
                        .set("update_time", DateUtils.parseDate(DateUtils.getTime()))
                        .set("update_by", SecurityUtils.getUserId())
                        .eq("id", id)
                        .eq("type",2));
                break;
            case "8":
                //文章评论
                communityArticleCommentService.update(new UpdateWrapper<CommunityArticleComment>()
                        .set("is_read", true)
                        .in("id", id)
                        .set("update_time", DateUtils.parseDate(DateUtils.getTime()))
                        .set("update_by", SecurityUtils.getUserId()));
                break;
            case "9":
                //文章回复
                communityCommentReplyService.update(new UpdateWrapper<CommunityCommentReply>()
                        .set("is_read", true)
                        .in("id", id)
                        .set("update_time", DateUtils.parseDate(DateUtils.getTime()))
                        .set("update_by", SecurityUtils.getUserId()));;
                break;
            case "10":
                //文章点赞
                communityLikeService.update(new UpdateWrapper<CommunityLike>()
                        .set("is_read", true)
                        .in("id", id)
                        .set("update_time", DateUtils.parseDate(DateUtils.getTime()))
                        .set("update_by", SecurityUtils.getUserId()));;
                break;
        }
        return success();
    }

    /**
     * 用户删除通知
     *
     * @return 用户删除通知
     */
    @PutMapping("/deleteMsgRead")
    @ApiOperation("用户点击通知后,更新通知消息已读")
    public AjaxResult deleteMsgRead(String type, String id) {
        switch (type) {
            case "0":
                //文章推荐
                communityArticleRecommendServiceImpl.update(new UpdateWrapper<CommunityArticleRecommend>()
                        .set("is_notice", true)
                        .set("update_time", DateUtils.parseDate(DateUtils.getTime()))
                        .set("update_by", SecurityUtils.getUserId())
                        .eq("id", id));
                break;
            case "1":
                //文章收藏
                articleCollectService.update(new UpdateWrapper<CommunityArticleCollect>()
                        .set("is_notice", true)
                        .set("update_time", DateUtils.parseDate(DateUtils.getTime()))
                        .set("update_by", SecurityUtils.getUserId())
                        .eq("id", id));
                break;
            case "2":
                //评论点赞
                commentLikeService.update(new UpdateWrapper<CommunityCommentLike>()
                        .set("is_notice", true)
                        .set("update_time", DateUtils.parseDate(DateUtils.getTime()))
                        .set("update_by", SecurityUtils.getUserId())
                        .eq("id", id));
                break;
            case "3":
                //回复点赞
                commentLikeService.update(new UpdateWrapper<CommunityCommentLike>()
                        .set("is_notice", true)
                        .set("update_time", DateUtils.parseDate(DateUtils.getTime()))
                        .set("update_by", SecurityUtils.getUserId())
                        .eq("id", id));
                break;
            case "4":
                //新增关注已读
                userLikeService.update(new UpdateWrapper<CommunityUserLike>()
                        .set("is_notice", true)
                        .set("update_time", DateUtils.parseDate(DateUtils.getTime()))
                        .set("update_by", SecurityUtils.getUserId())
                        .eq("id", id));
                break;
            case "5":
                //文章@
                communityArticleAtService.update(new UpdateWrapper<CommunityArticleAt>()
                        .set("is_notice", true)
                        .set("update_time", DateUtils.parseDate(DateUtils.getTime()))
                        .set("update_by", SecurityUtils.getUserId())
                        .eq("id", id)
                        .eq("type",0));
                break;
            case "6":
                //文章评论@
                communityArticleAtService.update(new UpdateWrapper<CommunityArticleAt>()
                        .set("is_notice", true)
                        .set("update_time", DateUtils.parseDate(DateUtils.getTime()))
                        .set("update_by", SecurityUtils.getUserId())
                        .eq("id", id)
                        .eq("type",1));
                break;
            case "7":
                //文章回复@
                communityArticleAtService.update(new UpdateWrapper<CommunityArticleAt>()
                        .set("is_notice", true)
                        .set("update_time", DateUtils.parseDate(DateUtils.getTime()))
                        .set("update_by", SecurityUtils.getUserId())
                        .eq("id", id)
                        .eq("type",2));
                break;
            case "8":
                //文章评论
                communityArticleCommentService.update(new UpdateWrapper<CommunityArticleComment>()
                        .set("is_notice", true)
                        .in("id", id)
                        .set("update_time", DateUtils.parseDate(DateUtils.getTime()))
                        .set("update_by", SecurityUtils.getUserId()));
                break;
            case "9":
                //文章回复
                communityCommentReplyService.update(new UpdateWrapper<CommunityCommentReply>()
                        .set("is_notice", true)
                        .in("id", id)
                        .set("update_time", DateUtils.parseDate(DateUtils.getTime()))
                        .set("update_by", SecurityUtils.getUserId()));;
                break;
            case "10":
                //文章点赞
                communityLikeService.update(new UpdateWrapper<CommunityLike>()
                        .set("is_notice", true)
                        .in("id", id)
                        .set("update_time", DateUtils.parseDate(DateUtils.getTime()))
                        .set("update_by", SecurityUtils.getUserId()));;
                break;
        }
        return success();
    }

}
