package com.ruoyi.generator.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.constant.CacheConstants;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.core.text.Convert;
import com.ruoyi.common.exception.user.ProjectException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServletUtils;
import com.ruoyi.generator.domain.Community.*;
import com.ruoyi.generator.mapper.community.CommunityReportDataMapper;
import com.ruoyi.generator.mapper.community.CommunityReportUserMapper;
import com.ruoyi.generator.service.ICommunityArticleService;
import com.ruoyi.generator.service.ICommunityCommentReplyService;
import com.ruoyi.generator.service.ICommunityReportUserService;
import com.ruoyi.generator.vo.CommunityReportPunishVo;
import com.ruoyi.generator.vo.CommunityReportUserVo;
import com.ruoyi.generator.vo.CommunityUserReportVo;
import com.ruoyi.system.domain.SysUserOnline;
import com.ruoyi.system.mapper.SysUserMapper;
import com.ruoyi.system.service.ISysUserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * 社区设置管理
 *
 * @author fangQing
 */
@Api(tags = "社区举报管理")
@RestController
@RequestMapping("/community/report")
public class CommunityReportController {


    @Autowired
    private CommunityReportDataMapper communityReportDataMapper;

    @Autowired
    private ICommunityReportUserService communityReportUserService;

    @Autowired
    private CommunityReportUserMapper communityReportUserMapper;

    @Autowired
    private ISysUserService userService;

    @Autowired
    private ICommunityArticleService communityArticleService;

    @Autowired
    private ICommunityCommentReplyService communityCommentReplyService;

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private RedisCache redisCache;

    /**
     * 获取举报类型信息
     *
     * @param
     * @return 获取举报类型信息
     */
    @ApiOperation("获取举报类型信息")
    @GetMapping("/reportMessage")
    //@Anonymous
    public AjaxResult reportTypes(String type) {

        int pageNum = Convert.toInt(ServletUtils.getParameter("pageNum"), 1);
        int pageSize = Convert.toInt(ServletUtils.getParameter("pageSize"), 10);
        //int offset = (pageNum - 1) * pageSize;
        Page<CommunityReportData> page = new Page<>(pageNum, pageSize);

        List<CommunityReportData> communityReportData = null;
        try {
            QueryWrapper<CommunityReportData> queryWrapper = new QueryWrapper<CommunityReportData>()
                    .eq("type", type)
                    .and((wrapper) -> {
                        wrapper.ne("is_delete", true).or().isNull("is_delete");
                    });
            communityReportData = communityReportDataMapper.selectPage(page, queryWrapper).getRecords();

        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }

        return AjaxResult.success(communityReportData);
    }


    /**
     * 用户举报
     *
     * @param
     * @return 用户举报
     */
    @ApiOperation("用户举报")
    @PostMapping("/reportTypes")
    //@Anonymous
    public AjaxResult reportMessage(@RequestBody CommunityReportUser communityReportUser) {
        if (Objects.isNull(communityReportUser.getUserId())) {
            return AjaxResult.error("请传被举报人ID!");
        }
        try {
            communityReportUserService.insertCommunityReportUser(communityReportUser);
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success("举报成功");
    }


    /**
     * 管理员查看界面
     *
     * @param id     明细ID
     * @param type   举报类型
     * @param name   用户名称
     * @param reason 举报理由
     * @param userId 用户ID
     * @return
     */
    @ApiOperation("管理员查看界面")
    @GetMapping("/adminReport")
    //@Anonymous
    public AjaxResult adminReport(Long id, Long type, String name, String reason, String userId) {
        List<CommunityReportUserVo> communityReportUsers = null;
        int total = 0;
        try {
            int pageNum = Convert.toInt(ServletUtils.getParameter("pageNum"), 1);
            int pageSize = Convert.toInt(ServletUtils.getParameter("pageSize"), 10);
            int offset = (pageNum - 1) * pageSize;
            int searchType = Convert.toInt(ServletUtils.getParameter("searchType"), 0);
            communityReportUsers = communityReportUserMapper.selectCommunityReportUser(offset, pageSize, searchType, id, type, name, reason, userId);

            total = communityReportUserMapper.selectList(new QueryWrapper<CommunityReportUser>()
                    .and((wrapper) -> {
                        wrapper.ne("is_delete", true).or().isNull("is_delete");
                    })).size();


        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.successByCounts(communityReportUsers, total);
    }

    /**
     * 管理员处罚
     *
     * @param
     * @return
     */
    @ApiOperation("管理员处罚")
    @PostMapping("/reportPunish")
    //@Anonymous
    public AjaxResult adminReportPunish(@RequestBody CommunityReportPunishVo communityReportPunishVo) {
        if (communityReportPunishVo.getId() == null) {
            return AjaxResult.error("ID不能为空");
        }
        List<CommunityReportUserVo> communityReportUsers = null;
        try {
            //判断已经处罚过的数据 不进行处理
            communityReportUsers = communityReportUserMapper
                    .selectCommunityReportUser(0, 100, 0, communityReportPunishVo.getId(), null, null, null, null);
            if (!communityReportUsers.isEmpty()) {
                CommunityReportUserVo firstUser = communityReportUsers.get(0);
                Long updateBy = firstUser.getUpdateBy();
                if (updateBy != null) {
                    return AjaxResult.success("当前举报数据已经被处理！");
                }
            }

            String disposalType = communityReportPunishVo.getDisposalType().toString(); //封禁类型
            Long disposalTime = communityReportPunishVo.getDisposalTime();              //封禁时间
            String dispositionReason = communityReportPunishVo.getDispositionReason();  //封禁理由


            //判断当前用户有没有被封禁或者禁言
            SysUser sysUser = sysUserMapper.selectUserById(communityReportPunishVo.getUserId());
            String userState = sysUser.getUserState();
            Date reportTime = sysUser.getReportTime();
            Date date = DateUtils.parseDate(DateUtils.getTime());
            if (reportTime != null && reportTime.after(date)) {
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-dd-MM HH:mm:ss");  // 修改日期格式
                String formattedReportTime = sdf.format(reportTime); // 格式化日期
                if ("1".equals(userState)) {
                    return AjaxResult.success("该用户已经被封号! 截止日期为：" + formattedReportTime);
                } else if ("2".equals(userState) && "2".equals(disposalType)) {
                    return AjaxResult.success("该用户已经被禁言! 截止日期为：" + formattedReportTime);
                }
            }

            //计算封禁的时间
            Date futureDate = null;
            if (disposalTime != null && disposalTime >= 0) {
                long millisecondsToAdd = TimeUnit.MINUTES.toMillis(disposalTime); //转换时间
                long currentTimeMillis = date.getTime();
                long futureTimeMillis = currentTimeMillis + millisecondsToAdd;
                futureDate = new Date(futureTimeMillis); //封禁时间
            }
            //更新个人信息表
            SysUser user = new SysUser();
            user.setUserId(communityReportPunishVo.getUserId());
            user.setUserState(String.valueOf(communityReportPunishVo.getDisposalType()));
            user.setReportTime(futureDate);

            //更新处罚记录表
            CommunityReportUser reportUser = new CommunityReportUser();
            reportUser.setId(communityReportPunishVo.getId());
            reportUser.setUserState(disposalType);
            reportUser.setPunishContent(dispositionReason);
            reportUser.setPunishTime(disposalTime);
            reportUser.setUpdateBy(SecurityUtils.getUserId());
            reportUser.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
            reportUser.setClosingTime(futureDate);

            Long replyId = communityReportPunishVo.getReplyId();
            switch (disposalType) {
                case "0": //不处理
                    break;
                case "1": //封号
                    //判断当前用户是否已经被封号
                    userService.updateUserProfile(user);
                    communityReportUserMapper.updateById(reportUser);
                    //清退token
                    Collection<String> keys = redisCache.keys(CacheConstants.LOGIN_TOKEN_KEY + "*");
                    List<SysUserOnline> userOnlineList = new ArrayList<SysUserOnline>();
                    for (String key : keys)
                    {
                        LoginUser loginUser = redisCache.getCacheObject(key);
                        if (loginUser.getUserId().equals(user.getUserId())) {
                            //移除用户已登录情况
                            redisCache.deleteObject(CacheConstants.LOGIN_TOKEN_KEY + key);
                        }
                    }
                    break;
                case "2": //禁言
                    userService.updateUserProfile(user);
                    communityReportUserMapper.updateById(reportUser);
                    break;
                case "3": //静置
                    CommunityArticle communityArticle = new CommunityArticle();
                    communityArticle.setIsDraft(true);
                    communityArticleService.update(communityArticle, new UpdateWrapper<CommunityArticle>().eq("id", communityReportPunishVo.getArticleId()));
                    communityReportUserMapper.updateById(reportUser);
                    break;
                case "4": //删除
                    if (replyId != null) {
                        CommunityCommentReply communityCommentReply = new CommunityCommentReply();
                        communityCommentReply.setId(Long.valueOf(communityReportPunishVo.getReplyId()));
                        communityCommentReply.setDelete(true);
                        communityCommentReply.setUpdateBy(SecurityUtils.getUserId());
                        communityCommentReply.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
                        communityCommentReplyService.updateById(communityCommentReply);
                        communityReportUserMapper.updateById(reportUser);
                    } else {
                        CommunityArticleComment communityArticleComment = new CommunityArticleComment();
                        communityArticleComment.setId(Long.valueOf(communityReportPunishVo.getCommentId()));
                        communityArticleComment.setDelete(true);
                        communityArticleComment.setUpdateBy(SecurityUtils.getUserId());
                        communityArticleComment.setUpdateTime(DateUtils.parseDate(DateUtils.getTime()));
                        communityArticleService.deleteComment(communityArticleComment);
                        communityReportUserMapper.updateById(reportUser);
                    }
                    //有评论也有回复
                    break;
            }

        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success(communityReportPunishVo);
    }


    /**
     * 查询用户是否被封禁
     *
     * @param
     * @return
     */
    @ApiOperation("查询用户是否被封禁")
    @GetMapping("/reportUser")
    //@Anonymous
    public AjaxResult adminReportPunish(Long userId) {
        if (Objects.isNull(userId)) {
            return AjaxResult.error("userId不能为空");
        }
        System.out.println(userId);
        long result = 0L;
        CommunityUserReportVo communityUserReportVo = new CommunityUserReportVo();
        try {

            SysUser sysUser = sysUserMapper.selectUserById(userId);
            communityUserReportVo.setUserId(sysUser.getUserId());
            communityUserReportVo.setUserState(sysUser.getUserState());
            Date reportTime = sysUser.getReportTime();
            communityUserReportVo.setReportTime(reportTime);

            Date date = DateUtils.parseDate(DateUtils.getTime());
            //封禁时间
            if (reportTime != null) {
                long diffInMillis = reportTime.getTime() - date.getTime();
                long diffInMinutes = TimeUnit.MILLISECONDS.toMinutes(diffInMillis);
                result = Math.max(0, diffInMinutes);
            }
            communityUserReportVo.setMinutes(result);


            //计算封禁时间如果小于当前日期 则解封
            if (reportTime != null && reportTime.before(date)) {
                SysUser user = new SysUser();
                user.setUserId(userId);
                user.setUserState("0");
                //判断当前用户是否已经被封号
                userService.updateUserProfile(user);
            }

        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new ProjectException();
        }
        return AjaxResult.success(communityUserReportVo);
    }
}
