package com.ruoyi.generator.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.generator.domain.Community.*;
import com.ruoyi.generator.mapper.community.CommunityArticleCommentMapper;
import com.ruoyi.generator.mapper.community.CommunityArticleMapper;
import com.ruoyi.generator.mapper.community.CommunityCommentReplyMapper;
import com.ruoyi.generator.mapper.community.CommunityUserLikeMapper;
import com.ruoyi.generator.vo.CommunityUserLikeVo;
import com.ruoyi.system.mapper.SysUserMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.DecimalFormat;
import java.time.LocalDate;
import java.time.ZoneId;
import java.time.temporal.ChronoUnit;
import java.time.temporal.Temporal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author fangqing
 * @date 2024/12/30 15:26
 */
@Service
public class CommunityAccompanyImpl implements ICommunityAccompanyService {


    @Autowired
    private SysUserMapper userMapper;

    @Autowired
    private CommunityArticleMapper communityArticleMapper;

    @Autowired
    private CommunityUserLikeMapper communityUserLikeMapper;

    @Autowired
    private CommunityArticleCommentMapper communityArticleCommentMapper;

    @Autowired
    private CommunityCommentReplyMapper communityCommentReplyMapper;

    @Override
    public CommunityAccompany selectCommunityAccompanyById(Long userId) {
        DecimalFormat df = new DecimalFormat("#.00");
        CommunityAccompany communityAccompany = new CommunityAccompany();

        //获取当前用户信息 天数
        SysUser sysUsers = userMapper.selectOne( new QueryWrapper<SysUser>().select("create_time").eq("user_id", userId).and((wrapper) -> {wrapper.ne("status", true).or().isNull("status");}));

        if (sysUsers == null){
            return  communityAccompany;
        }
        Date createTime = sysUsers.getCreateTime();
        // 将 Date 对象转换为 LocalDate 对象
        LocalDate localCreateTime = createTime.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
        // 获取当前日期
        LocalDate today = LocalDate.now();
        // 计算两个日期之间的天数差
        long daysDifference = ChronoUnit.DAYS.between(localCreateTime, today);
        communityAccompany.setAccompanyDays(daysDifference);

        //查询当前用户下文章的数量
        List<CommunityArticle> communityArticles = communityArticleMapper.selectList(new QueryWrapper<CommunityArticle>()
                .eq("create_by", userId)
                .and((wrapper) -> {
                    wrapper.ne("is_delete", true).or().isNull("is_delete");
                }));
        //如果用户文章数量不为空则执行以下的操作
        if (communityArticles != null){

            //设置用户创作数
            communityAccompany.setCreationCount((long) communityArticles.size());

            //计算小于自己粉丝的有多少人
            List<CommunityArticle> communityArticlesLess = new ArrayList<>(communityArticleMapper.selectList(new QueryWrapper<CommunityArticle>()
                    .select("user_id", "COUNT(1) as count")
                    .and((wrapper) -> {
                        wrapper.ne("is_delete", true).or().isNull("is_delete");
                    })
                    .groupBy("user_id")
                    .having("COUNT(*) <= {0}", (long) communityArticles.size())
                    .orderByDesc("count"))) ;

            //计算所有文章的数量
            List<CommunityArticle> communityArticlesTotal = new ArrayList<>(communityArticleMapper.selectList(new QueryWrapper<CommunityArticle>()
                    .select("user_id", "COUNT(1) as count")
                    .and((wrapper) -> {
                        wrapper.ne("is_delete", true).or().isNull("is_delete");
                    })
                    .groupBy("user_id")
                    .orderByDesc("count")));

            //计算创作数的百分比 计算百分比
            Double CreationCountPercent = Double.valueOf(df.format((double) communityArticlesLess.size() / communityArticlesTotal.size() * 100));
            communityAccompany.setCreationCountPercent(CreationCountPercent);


            //计算用户阅览量
            long read = communityArticles.stream()
                    .mapToLong(article -> article.getPageViews()).sum();
            communityAccompany.setRead(read);


            //计算阅览量百分比
            List<CommunityArticle> communityArticlesReadSumLess = new ArrayList<>(communityArticleMapper.selectList(new QueryWrapper<CommunityArticle>()
                    .select("user_id","sum(page_views) page_views","COUNT(1) as count")
                    .and((wrapper) -> {
                        wrapper.ne("is_delete", true).or().isNull("is_delete");
                    })
                    .groupBy("user_id")
                    .having("page_views <= {0}", read )
                    .orderByDesc("count"))) ;

            Double ReadPercent = Double.valueOf(df.format((double)  communityArticlesReadSumLess.size() / communityArticlesTotal.size() * 100));
            communityAccompany.setReadPercent(ReadPercent);

        }


        //粉丝数
        Long fans = communityUserLikeMapper.selectCount(new QueryWrapper<CommunityUserLike>()
                .eq("like_user_id", userId));


        communityAccompany.setFan(fans);
        //计算小于自己粉丝的有多少人
        List<CommunityUserLike> userLikeFan = new ArrayList<>(communityUserLikeMapper.selectList(new QueryWrapper<CommunityUserLike>()
                .select("like_user_id", "COUNT(1) as count")
                .groupBy("like_user_id")
                .having("COUNT(*) <= {0}", fans)
                .orderByDesc("count"))) ;

        //计算总人数
        List<CommunityUserLike> userLikeFans = new ArrayList<>(communityUserLikeMapper.selectList(new QueryWrapper<CommunityUserLike>()
                .select("like_user_id", "COUNT(1) as count")
                .groupBy("like_user_id")));
        //粉丝数占比
        Double  ReadPercent = Double.valueOf(df.format((double) userLikeFan.size() / userLikeFans.size() * 100));
        communityAccompany.setFanPercent(ReadPercent);


        //查询评论数
        /*//一级评论
        Long comment = communityArticleCommentMapper.selectCount(new QueryWrapper<CommunityArticleComment>()
                .eq("user_id", userId)
                .and((wrapper) -> {
                    wrapper.ne("is_delete", true).or().isNull("is_delete");
                })
        );
        //二级评论
        Long commentReply = communityCommentReplyMapper.selectCount(new QueryWrapper<CommunityCommentReply>()
                .eq("user_id", userId)
                .and((wrapper) -> {
                    wrapper.ne("is_delete", true).or().isNull("is_delete");
                })
        );*/
        //查询哪几篇文章是我发布的
        List<Long> articleIds = communityArticles.stream().map(item -> item.getId()).collect(Collectors.toList());
        System.out.println(articleIds);
        Integer commentSum = 0;
        //查询评论数 别人评论我的数量 首先自己得有文章才能有评论
        if (articleIds.size() > 0 ) {
            commentSum = communityArticleCommentMapper.queryCommentReplyCount(articleIds);
        }
        communityAccompany.setComment(Long.valueOf(commentSum));

        //评论数占比
        if (commentSum > 0 ){
            Integer commentless = communityArticleCommentMapper.queryCommentReplyCountPerson(commentSum);
            Integer commentCount = communityArticleCommentMapper.queryCommentReplyCountPerson(0);
            Double  commentPercent = Double.valueOf(df.format((double) commentless / commentCount * 100));
            communityAccompany.setCommentPercent(commentPercent);
        }

        //用户文章 和 评论收货多少点赞
        Integer likeCount = Optional.ofNullable(communityArticleCommentMapper.queryCommentReplyLikeCount(userId))
                .orElse(0);
        communityAccompany.setLike(Long.valueOf(likeCount));

        if (likeCount >0 ){
            Integer likeless = communityArticleCommentMapper.queryCommentReplyLikeCountPerson(likeCount);
            Integer likesCount = communityArticleCommentMapper.queryCommentReplyLikeCountPerson(0);
            Double likePercent = Double.valueOf(df.format((double) likeless / likesCount * 100));
            communityAccompany.setLikePercent(likePercent);
        }
        //

        return communityAccompany;
    }


}
